class Review:

	Clayful = None
	name = 'Review'
	path = 'products/reviews'

	@staticmethod
	def config(clayful):

		Review.Clayful = clayful

		return Review

	@staticmethod
	def list(*args):

		return Review.Clayful.call_api({
			'model_name':       Review.name,
			'method_name':      'list',
			'http_method':      'GET',
			'path':             '/v1/products/reviews',
			'params':           (),
			'args':             args
		})

	@staticmethod
	def count(*args):

		return Review.Clayful.call_api({
			'model_name':       Review.name,
			'method_name':      'count',
			'http_method':      'GET',
			'path':             '/v1/products/reviews/count',
			'params':           (),
			'args':             args
		})

	@staticmethod
	def list_published(*args):

		return Review.Clayful.call_api({
			'model_name':       Review.name,
			'method_name':      'list_published',
			'http_method':      'GET',
			'path':             '/v1/products/reviews/published',
			'params':           (),
			'args':             args
		})

	@staticmethod
	def get(*args):

		return Review.Clayful.call_api({
			'model_name':       Review.name,
			'method_name':      'get',
			'http_method':      'GET',
			'path':             '/v1/products/reviews/{reviewId}',
			'params':           ('reviewId', ),
			'args':             args
		})

	@staticmethod
	def count_published(*args):

		return Review.Clayful.call_api({
			'model_name':       Review.name,
			'method_name':      'count_published',
			'http_method':      'GET',
			'path':             '/v1/products/reviews/published/count',
			'params':           (),
			'args':             args
		})

	@staticmethod
	def get_published(*args):

		return Review.Clayful.call_api({
			'model_name':       Review.name,
			'method_name':      'get_published',
			'http_method':      'GET',
			'path':             '/v1/products/reviews/published/{reviewId}',
			'params':           ('reviewId', ),
			'args':             args
		})

	@staticmethod
	def create(*args):

		return Review.Clayful.call_api({
			'model_name':       Review.name,
			'method_name':      'create',
			'http_method':      'POST',
			'path':             '/v1/products/reviews',
			'params':           (),
			'args':             args
		})

	@staticmethod
	def create_for_me(*args):

		return Review.Clayful.call_api({
			'model_name':       Review.name,
			'method_name':      'create_for_me',
			'http_method':      'POST',
			'path':             '/v1/me/products/reviews',
			'params':           (),
			'args':             args
		})

	@staticmethod
	def flag(*args):

		return Review.Clayful.call_api({
			'model_name':       Review.name,
			'method_name':      'flag',
			'http_method':      'POST',
			'path':             '/v1/products/reviews/{reviewId}/flags',
			'params':           ('reviewId', ),
			'args':             args
		})

	@staticmethod
	def flag_for_me(*args):

		return Review.Clayful.call_api({
			'model_name':       Review.name,
			'method_name':      'flag_for_me',
			'http_method':      'POST',
			'path':             '/v1/me/products/reviews/{reviewId}/flags',
			'params':           ('reviewId', ),
			'without_payload':  True,
			'args':             args
		})

	@staticmethod
	def helped(*args):

		return Review.Clayful.call_api({
			'model_name':       Review.name,
			'method_name':      'helped',
			'http_method':      'POST',
			'path':             '/v1/products/reviews/{reviewId}/helped/{upDown}',
			'params':           ('reviewId', 'upDown', ),
			'args':             args
		})

	@staticmethod
	def helped_for_me(*args):

		return Review.Clayful.call_api({
			'model_name':       Review.name,
			'method_name':      'helped_for_me',
			'http_method':      'POST',
			'path':             '/v1/me/products/reviews/{reviewId}/helped/{upDown}',
			'params':           ('reviewId', 'upDown', ),
			'without_payload':  True,
			'args':             args
		})

	@staticmethod
	def increase_metafield(*args):

		return Review.Clayful.call_api({
			'model_name':       Review.name,
			'method_name':      'increase_metafield',
			'http_method':      'POST',
			'path':             '/v1/products/reviews/{reviewId}/meta/{field}/inc',
			'params':           ('reviewId', 'field', ),
			'args':             args
		})

	@staticmethod
	def push_to_metafield(*args):

		return Review.Clayful.call_api({
			'model_name':       Review.name,
			'method_name':      'push_to_metafield',
			'http_method':      'POST',
			'path':             '/v1/products/reviews/{reviewId}/meta/{field}/push',
			'params':           ('reviewId', 'field', ),
			'args':             args
		})

	@staticmethod
	def pull_from_metafield(*args):

		return Review.Clayful.call_api({
			'model_name':       Review.name,
			'method_name':      'pull_from_metafield',
			'http_method':      'POST',
			'path':             '/v1/products/reviews/{reviewId}/meta/{field}/pull',
			'params':           ('reviewId', 'field', ),
			'args':             args
		})

	@staticmethod
	def update(*args):

		return Review.Clayful.call_api({
			'model_name':       Review.name,
			'method_name':      'update',
			'http_method':      'PUT',
			'path':             '/v1/products/reviews/{reviewId}',
			'params':           ('reviewId', ),
			'args':             args
		})

	@staticmethod
	def update_for_me(*args):

		return Review.Clayful.call_api({
			'model_name':       Review.name,
			'method_name':      'update_for_me',
			'http_method':      'PUT',
			'path':             '/v1/me/products/reviews/{reviewId}',
			'params':           ('reviewId', ),
			'args':             args
		})

	@staticmethod
	def delete(*args):

		return Review.Clayful.call_api({
			'model_name':       Review.name,
			'method_name':      'delete',
			'http_method':      'DELETE',
			'path':             '/v1/products/reviews/{reviewId}',
			'params':           ('reviewId', ),
			'args':             args
		})

	@staticmethod
	def delete_for_me(*args):

		return Review.Clayful.call_api({
			'model_name':       Review.name,
			'method_name':      'delete_for_me',
			'http_method':      'DELETE',
			'path':             '/v1/me/products/reviews/{reviewId}',
			'params':           ('reviewId', ),
			'args':             args
		})

	@staticmethod
	def cancel_flag_for_me(*args):

		return Review.Clayful.call_api({
			'model_name':       Review.name,
			'method_name':      'cancel_flag_for_me',
			'http_method':      'DELETE',
			'path':             '/v1/me/products/reviews/{reviewId}/flags',
			'params':           ('reviewId', ),
			'args':             args
		})

	@staticmethod
	def delete_metafield(*args):

		return Review.Clayful.call_api({
			'model_name':       Review.name,
			'method_name':      'delete_metafield',
			'http_method':      'DELETE',
			'path':             '/v1/products/reviews/{reviewId}/meta/{field}',
			'params':           ('reviewId', 'field', ),
			'args':             args
		})

	@staticmethod
	def cancel_flag(*args):

		return Review.Clayful.call_api({
			'model_name':       Review.name,
			'method_name':      'cancel_flag',
			'http_method':      'DELETE',
			'path':             '/v1/products/reviews/{reviewId}/flags/{customerId}',
			'params':           ('reviewId', 'customerId', ),
			'args':             args
		})

	@staticmethod
	def cancel_helped_for_me(*args):

		return Review.Clayful.call_api({
			'model_name':       Review.name,
			'method_name':      'cancel_helped_for_me',
			'http_method':      'DELETE',
			'path':             '/v1/me/products/reviews/{reviewId}/helped/{upDown}',
			'params':           ('reviewId', 'upDown', ),
			'args':             args
		})

	@staticmethod
	def cancel_helped(*args):

		return Review.Clayful.call_api({
			'model_name':       Review.name,
			'method_name':      'cancel_helped',
			'http_method':      'DELETE',
			'path':             '/v1/products/reviews/{reviewId}/helped/{upDown}/{customerId}',
			'params':           ('reviewId', 'upDown', 'customerId', ),
			'args':             args
		})




import unittest
from clayful import Clayful
from clayful.exception import ClayfulException

# Define mock requester class for tests
class MockRequester():

	def __init__(self, return_value = None):

		self.return_value = return_value

	def request(self, request_option):

		if isinstance(self.return_value, Exception):

			raise self.return_value

		else:

			return self.return_value


class ClayfulMainModuleTest(unittest.TestCase):

	def setUp(self):

		Clayful.listeners['request'] = []
		Clayful.listeners['response'] = []


	def test__default_options_for_api_client(self):

		self.assertEqual(Clayful.base_url, 'https://api.clayful.io')
		self.assertEqual(Clayful.default_headers, {
			'Accept-Encoding': 'gzip',
			'User-Agent':      'clayful-python',
			'Clayful-SDK':     'clayful-python'
		})
		self.assertTrue(callable(Clayful.plugins['request']))
		self.assertEqual(str(type(Clayful.Brand)), "<type 'classobj'>")
		self.assertEqual(Clayful.Brand.Clayful, Clayful)


	def test_options_to_headers(self):

		result1 = Clayful.options_to_headers({})
		result2 = Clayful.options_to_headers({
			'language':       'ko',
			'currency':       'KRW',
			'time_zone':      'Asia/Seoul',
			'client':         'client_token',
			'customer':       'customer_token',
			'reCAPTCHA':      'response',
			'debug_language': 'ko',
			'headers':        {
				'X-Extra': 'Extra'
			}
		})

		self.assertEqual(result1, {})
		self.assertEqual(result2, {
			'Accept-Language':        'ko',
			'Accept-Currency':        'KRW',
			'Accept-Time-Zone':       'Asia/Seoul',
			'Authorization':          'Bearer client_token',
			'Authorization-Customer': 'customer_token',
			'reCAPTCHA-Response':     'response',
			'Accept-Debug-Language':  'ko',
			'X-Extra':                'Extra'
		})


	def test_get_api_endpoint(self):

		self.assertEqual(Clayful.get_endpoint('/v1/products'), Clayful.base_url + '/v1/products')


	def test_normalize_query_values(self):

		result = Clayful.normalize_query_values({
			'string':  'string',
			'number':  100,
			'boolean': True,
			'special': ' +&',
		})

		self.assertEqual(result, {
			'string':  'string',
			'number':  '100',
			'boolean': 'true',
			'special': ' +&',
		})


	def test_extract_api_request_arguments(self):

		def build_get_delete_test_cases(method):

			return [
				{
					'options': {
						'http_method': method,
						'path':        '/v1/products', # no params
						'params':      (),
						'args':        () # no args
					},
					'result':   {
						'http_method': method,
						'request_url': '/v1/products',
						'payload':     None,
						'query':       {},
						'headers':     {},
						'meta':        {}, # default meta
					}
				},
				{
					'options':  {
						'http_method': method,
						'path':        '/v1/products/{productId}',
						'params':      ('productId', ),
						'args':        (
							'pid', # param1
							{      # queryHeaders
								'query':    { 'raw': True, 'originalPrice>': 1000, 'type': 'custom' },
								'language': 'en'
							},
						)
					},
					'result':   {
						'http_method': method,
						'request_url': '/v1/products/pid',
						'payload':     None,
						'query':       { 'raw': 'true', 'originalPrice>': '1000', 'type': 'custom' }, # stringified values
						'headers':     { 'Accept-Language': 'en' },
						'meta':        {}, # default meta
					}
				},
			]


		def build_post_put_test_cases(method):

			return [
				{
					'options':  {
						'http_method': method,
						'path':        '/v1/store', # no params
						'params':      (),
						'args':        (
							{ 'slug': 'new-slug' },  # payload
						)
					},
					'result':   {
						'http_method': method,
						'request_url': '/v1/store',
						'payload':     { 'slug': 'new-slug' },
						'query':       {},
						'headers':     {},
						'meta':        {},
					}
				},
				{
					'options':  {
						'http_method': method,
						'path':        '/v1/products/{productId}',
						'params':      ('productId', ),
						'args':        (
							'pid', # param1
							{      # payload
								'slug': 'new-slug'
							},
							{      # queryHeaders
								'query':    { 'raw': True },
								'language': 'en',
								'meta':     { 'data': 'data' }
							},
						)
					},
					'result':   {
						'http_method': method,
						'request_url': '/v1/products/pid',
						'payload':     { 'slug': 'new-slug' },
						'query':       { 'raw': 'true' }, # stringified values
						'headers':     { 'Accept-Language': 'en' },
						'meta':        { 'data': 'data' }
					}
				},
				{
					'options':  {
						'http_method':     method,
						'path':            '/v1/me/products/reviews/{reviewId}/flag',
						'params':          ('reviewId', ),
						'without_payload': True, # no payload
						'args':            (
							'rid', # param1
							{      # queryHeaders
								'query':    { 'raw': True },
								'language': 'en',
								'meta':     { 'data': 'data' }
							},
						)
					},
					'result':   {
						'http_method': method,
						'request_url': '/v1/me/products/reviews/rid/flag',
						'payload':     None,
						'query':       { 'raw': 'true' }, # stringified values
						'headers':     { 'Accept-Language': 'en' },
						'meta':        { 'data': 'data' }
					}
				},
			]

		cases = build_get_delete_test_cases('GET') + build_get_delete_test_cases('DELETE') + build_post_put_test_cases('POST') + build_post_put_test_cases('PUT')

		for c in cases:

			extracted = Clayful.extract_request_arguments(c['options']);

			self.assertEqual(extracted, c['result'])

	def test_call_api_success(self):

		triggered = {
			'request':  False,
			'response': False,
		}

		# Will return { 'response': True } as mock response
		mock_requester = MockRequester({ 'response': True })

		Clayful.install('request', mock_requester.request)

		def request_callback(detail):
			self.assertEqual(detail, {
				'model_name':     'Brand',
				'method_name':    'query',
				'http_method':    'GET',
				'request_url':    Clayful.base_url + '/v1/brands',
				'query':          {},
				'headers':        {
					'Accept-Encoding': 'gzip',
					'User-Agent':      'clayful-python',
					'Accept-Currency': 'KRW',
					'Clayful-SDK':     'clayful-python'
				},
				'payload':        None,
				'uses_form_data': False,
				'meta':           {},
				'error':          None,
				'response':       None # not populated yet
			})

			# set a new data
			detail['meta']['request_called'] = True
			triggered['request'] = True

		def response_callback(detail):
			self.assertEqual(detail, {
				'model_name':     'Brand',
				'method_name':    'query',
				'http_method':    'GET',
				'request_url':    Clayful.base_url + '/v1/brands',
				'query':          {},
				'headers':        {
					'Accept-Encoding': 'gzip',
					'User-Agent':      'clayful-python',
					'Accept-Currency': 'KRW',
					'Clayful-SDK':     'clayful-python'
				},
				'payload':        None,
				'uses_form_data': False,
				'meta':           { 'request_called': True }, # check data
				'error':          None,
				'response':       { 'response': True } # populated
			})
			triggered['response'] = True

		Clayful.on('request', request_callback)
		Clayful.on('response', response_callback)

		self.assertEqual(Clayful.default_headers, {
			'Accept-Encoding': 'gzip',
			'User-Agent':      'clayful-python',
			'Clayful-SDK':     'clayful-python'
		})

		response = Clayful.call_api({
			'model_name':     'Brand',
			'method_name':    'query',
			'http_method':    'GET',
			'path':           '/v1/brands',
			'params':         (),
			'args':           ({ 'currency': 'KRW' }, )
		})

		self.assertEqual(response, { 'response': True })
		self.assertEqual(triggered, {
			'request':  True,
			'response': True,
		})

		self.assertEqual(Clayful.default_headers, {
			'Accept-Encoding': 'gzip',
			'User-Agent':      'clayful-python',
			'Clayful-SDK':     'clayful-python'
		})


	def test_call_api_with_form_data_success(self):

		triggered = {
			'request':  False,
			'response': False,
		}

		# Will return { 'response': True } as mock response
		mock_requester = MockRequester({ 'response': True })

		Clayful.install('request', mock_requester.request)

		def request_callback(detail):
			self.assertEqual(detail, {
				'model_name':     'Image',
				'method_name':    'create',
				'http_method':    'POST',
				'request_url':    Clayful.base_url + '/v1/images',
				'query':          {},
				'headers':        {
					'Accept-Encoding': 'gzip',
					'User-Agent':      'clayful-python',
					'Accept-Language': 'ko',
					'Clayful-SDK':     'clayful-python'
				},
				'payload':        { 'file': 'file' },
				'uses_form_data': True,
				'meta':           {},
				'error':          None,
				'response':       None # not populated yet
			})

			# set a new data
			detail['meta']['request_called'] = True
			triggered['request'] = True


		def response_callback(detail):
			self.assertEqual(detail, {
				'model_name':     'Image',
				'method_name':    'create',
				'http_method':    'POST',
				'request_url':    Clayful.base_url + '/v1/images',
				'query':          {},
				'headers':        {
					'Accept-Encoding': 'gzip',
					'User-Agent':      'clayful-python',
					'Accept-Language': 'ko',
					'Clayful-SDK':     'clayful-python'
				},
				'payload':        { 'file': 'file' },
				'uses_form_data': True,
				'meta':           { 'request_called': True },
				'error':          None,
				'response':       { 'response': True }
			})
			triggered['response'] = True

		Clayful.on('request', request_callback)
		Clayful.on('response', response_callback)

		self.assertEqual(Clayful.default_headers, {
			'Accept-Encoding': 'gzip',
			'User-Agent':      'clayful-python',
			'Clayful-SDK':     'clayful-python'
		})

		response = Clayful.call_api({
			'model_name':     'Image',
			'method_name':    'create',
			'http_method':    'POST',
			'path':           '/v1/images',
			'params':         (),
			'uses_form_data': True, # Test form data
			'args':           (
				{ 'file': 'file' },
				{ 'language': 'ko' },
			)
		})

		self.assertEqual(response, { 'response': True })
		self.assertEqual(triggered, {
			'request':  True,
			'response': True,
		})

		self.assertEqual(Clayful.default_headers, {
			'Accept-Encoding': 'gzip',
			'User-Agent':      'clayful-python',
			'Clayful-SDK':     'clayful-python'
		})


	def test_call_api_fail_with_clayful_exception(self):

		triggered = {
			'request':  False,
			'response': False,
		}

		error = ClayfulException()

		# Will throw ClayfulException
		mock_requester = MockRequester(error)

		Clayful.install('request', mock_requester.request)

		def request_callback(detail):
			self.assertEqual(detail, {
				'model_name':     'Brand',
				'method_name':    'query',
				'http_method':    'GET',
				'request_url':    Clayful.base_url + '/v1/brands',
				'query':          {},
				'headers':        {
					'Accept-Encoding': 'gzip',
					'User-Agent':      'clayful-python',
					'Accept-Currency': 'KRW',
					'Clayful-SDK':     'clayful-python'
				},
				'payload':        None,
				'uses_form_data': False,
				'meta':           {},
				'error':          None, # not populated yet
				'response':       None
			})

			# set a new data
			detail['meta']['request_called'] = True
			triggered['request'] = True


		def response_callback(detail):
			self.assertEqual(detail, {
				'model_name':     'Brand',
				'method_name':    'query',
				'http_method':    'GET',
				'request_url':    Clayful.base_url + '/v1/brands',
				'query':          {},
				'headers':        {
					'Accept-Encoding': 'gzip',
					'User-Agent':      'clayful-python',
					'Accept-Currency': 'KRW',
					'Clayful-SDK':     'clayful-python'
				},
				'payload':        None,
				'uses_form_data': False,
				'meta':           { 'request_called': True }, # check data
				'error':          error, # populated
				'response':       None
			})
			triggered['response'] = True

		Clayful.on('request', request_callback)
		Clayful.on('response', response_callback)

		try:

			response = Clayful.call_api({
				'model_name':     'Brand',
				'method_name':    'query',
				'http_method':    'GET',
				'path':           '/v1/brands',
				'params':         (),
				'args':           ({ 'currency': 'KRW' }, )
			})

			# Just to check ClayfulException raise
			self.assertEqual(1, 2)

		except ClayfulException as e:

			self.assertEqual(e, error)
			self.assertEqual(triggered, {
				'request':  True,
				'response': True, # should've been triggered
			})


	def test_call_api_fail_with_other_exception(self):

		triggered = {
			'request':  False,
			'response': False,
		}

		error = Exception()

		# Will throw ClayfulException
		mock_requester = MockRequester(error)

		Clayful.install('request', mock_requester.request)

		def request_callback(detail):
			triggered['request'] = True


		def response_callback(detail):
			triggered['response'] = True

		Clayful.on('request', request_callback)
		Clayful.on('response', response_callback)

		try:

			response = Clayful.call_api({
				'model_name':     'Brand',
				'method_name':    'query',
				'http_method':    'GET',
				'path':           '/v1/brands',
				'params':         (),
				'args':           ({ 'currency': 'KRW' }, )
			})

			# Just to check Exception raise
			self.assertEqual(1, 2)

		except Exception as e:

			self.assertEqual(e, error)
			self.assertEqual(triggered, {
				'request':  True,
				'response': False, # shouldn't have been triggered
			})


	def test_config_api_client(self):

		Clayful.config({
			'language':       'ko',
			'currency':       'KRW',
			'time_zone':      'Asia/Seoul',
			'client':         'client_token',
			'customer':       'customer_token',
			'reCAPTCHA':      'response',
			'debug_language': 'ko',
			'headers':        {
				'X-Extra': 'Extra'
			}
		})

		self.assertEqual(Clayful.default_headers, {
			'Accept-Encoding':        'gzip',
			'Accept-Language':        'ko',
			'Accept-Currency':        'KRW',
			'Accept-Time-Zone':       'Asia/Seoul',
			'Authorization':          'Bearer client_token',
			'Authorization-Customer': 'customer_token',
			'reCAPTCHA-Response':     'response',
			'Accept-Debug-Language':  'ko',
			'User-Agent':             'clayful-python',
			'Clayful-SDK':            'clayful-python',
			'X-Extra':                'Extra'
		})


	def test_install_request_middleware(self):

		def req():
			pass

		Clayful.install('request', req)

		self.assertEqual(Clayful.plugins['request'], req)


	def test_event_listeners(self):

		triggered = {
			'request':  None,
			'response': None
		}

		self.assertEqual(len(Clayful.listeners['request']), 0)
		self.assertEqual(len(Clayful.listeners['response']), 0)

		def request_callback(data):
			triggered['request'] = data

		def response_callback(data):
			triggered['response'] = data

		Clayful.on('request', request_callback)
		Clayful.on('response', response_callback)

		self.assertEqual(len(Clayful.listeners['request']), 1)
		self.assertEqual(len(Clayful.listeners['response']), 1)
		self.assertEqual(Clayful.listeners['request'][0], request_callback)
		self.assertEqual(Clayful.listeners['response'][0], response_callback)

		Clayful.trigger('request', 1)
		Clayful.trigger('response', 2)

		self.assertEqual(triggered, {
			'request':  1,
			'response': 2
		})

		Clayful.off('request', request_callback)
		Clayful.off('response', response_callback)

		self.assertEqual(len(Clayful.listeners['request']), 0)
		self.assertEqual(len(Clayful.listeners['response']), 0)


	def test_format_image_url(self):

		url = 'http://clayful.io'

		cases = [
			{
				'out':     url
			},
			{
				'out':     url,
				'options': {}
			},
			{
				'out':     url + '?width=120',
				'options': { 'width' : 120 }
			},
			{
				'out':     url + '?width=120&height=120',
				'options': { 'width' : 120, 'height' : 120 }
			}
		]

		for c in cases:

			options = c.get('options', None)

			if options is None:

				self.assertEqual(Clayful.format_image_url(url), c.get('out'))

			else:
				self.assertEqual(Clayful.format_image_url(url, options), c.get('out'))

	def test_format_number(self):

		cases = [
			{
				'in':  None,
				'out': ''
			},
			{
				'in':  0,
				'out': '0'
			},
			{
				'in':  10,
				'out': '10'
			},
			{
				'options': {},
				'in':      10,
				'out':     '10'
			},
			# precision tests
			{
				'in':  0.250,
				'out': '0.25'
			},
			{
				'options': { 'precision' : 0 },
				'in':      0,
				'out':     '0'
			},
			{
				'options': { 'precision' : 0 },
				'in':      1234567.25,
				'out':     '1234567'
			},
			{
				'options': { 'precision' : 1 },
				'in':      1234567.24,
				'out':     '1234567.2' # rounded
			},
			{
				'options': { 'precision' : 1 },
				'in':      1234567.25,
				'out':     '1234567.3' # rounded
			},
			{
				'options': { 'precision' : 2 },
				'in':      1234567.25,
				'out':     '1234567.25'
			},
			{
				'options': { 'precision' : 3 },
				'in':      1234567.25,
				'out':     '1234567.250'
			},
			{
				'options': { 'precision' : 0 },
				'in':      1234567,
				'out':     '1234567'
			},
			{
				'options': { 'precision' : 3 },
				'in':      1234567,
				'out':     '1234567.000'
			},
			# delimiter tests
			{
				'options': {
					'precision': 3
				},
				'in':  1234567.25,
				'out': '1234567.250'
			},
			{
				'options': {
					'precision': 3,
					'delimiter': {}
				},
				'in':  1234567.25,
				'out': '1234567.250'
			},
			{
				'options': {
					'precision': 3,
					'delimiter': {
						'thousands': ','
					}
				},
				'in':  1234567.25,
				'out': '1,234,567.250'
			},
			{
				'options': {
					'precision': 3,
					'delimiter': {
						'thousands': ' ',
						'decimal':   ','
					}
				},
				'in':  1234567.25,
				'out': '1 234 567,250'
			},
		]

		for c in cases:

			options = c.get('options', None)

			if options is None:

				self.assertEqual(Clayful.format_number(c.get('in')), c.get('out'))

			else:
				self.assertEqual(Clayful.format_number(c.get('in'), options), c.get('out'))


	def test_format_price(self):

		cases = [
			{
				'in':  None,
				'out': ''
			},
			{
				'in':  0,
				'out': '0'
			},
			{
				'options': {},
				'in':      0,
				'out':     '0'
			},
			{
				'in':  1234567.25,
				'out': '1234567.25'
			},
			{
				'options': {},
				'in':      1234567.25,
				'out':     '1234567.25'
			},
			{
				'options': {
					'symbol':    '$',
					'format':    '{symbol}{price}',
					'precision': 2,
					'delimiter': {
						'thousands': ',',
						'decimal':   '.'
					}
				},
				'in':  1234567.25,
				'out': '$1,234,567.25'
			},
			{
				'options': {
					'symbol':    'won',
					'format':    '{price} {symbol}',
					'precision': 0,
					'delimiter': {
						'thousands': ',',
						'decimal':   '.'
					}
				},
				'in':  1234567.25,
				'out': '1,234,567 won'
			}
		]

		for c in cases:

			options = c.get('options', None)

			if options is None:

				self.assertEqual(Clayful.format_price(c.get('in')), c.get('out'))

			else:
				self.assertEqual(Clayful.format_price(c.get('in'), options), c.get('out'))


import unittest
from clayful.exception import ClayfulException

class ClayfulExceptionTest(unittest.TestCase):

	def test_clayful_error_constructor(self):

		error = ClayfulException(
			'Brand',
			'get',
			400,
			{},
			'g-no-model',
			'my message',
			{}
		)

		self.assertEqual(error.is_clayful, True)
		self.assertEqual(error.model, 'Brand')
		self.assertEqual(error.method, 'get')
		self.assertEqual(error.status, 400)
		self.assertEqual(error.headers, {})
		self.assertEqual(error.code, 'g-no-model')
		self.assertEqual(error.message, 'my message')
		self.assertEqual(error.validation, {})

	def test_throw_clayful_error(self):

		try:

			raise ClayfulException(
				'Brand',
				'get',
				400,
				{},
				'g-no-model',
				'my message',
				{}
			)

		except ClayfulException as e:

			self.assertEqual(e.is_clayful, True)


class Downloadable:

	Clayful = None
	name = 'Downloadable'
	path = 'downloadables'

	@staticmethod
	def config(clayful):

		Downloadable.Clayful = clayful

		return Downloadable

	@staticmethod
	def list(*args):

		return Downloadable.Clayful.call_api({
			'model_name':       Downloadable.name,
			'method_name':      'list',
			'http_method':      'GET',
			'path':             '/v1/downloadables',
			'params':           (),
			'args':             args
		})

	@staticmethod
	def count(*args):

		return Downloadable.Clayful.call_api({
			'model_name':       Downloadable.name,
			'method_name':      'count',
			'http_method':      'GET',
			'path':             '/v1/downloadables/count',
			'params':           (),
			'args':             args
		})

	@staticmethod
	def get(*args):

		return Downloadable.Clayful.call_api({
			'model_name':       Downloadable.name,
			'method_name':      'get',
			'http_method':      'GET',
			'path':             '/v1/downloadables/{downloadableId}',
			'params':           ('downloadableId', ),
			'args':             args
		})

	@staticmethod
	def create_download_url(*args):

		return Downloadable.Clayful.call_api({
			'model_name':       Downloadable.name,
			'method_name':      'create_download_url',
			'http_method':      'POST',
			'path':             '/v1/downloadables/{downloadableId}/url',
			'params':           ('downloadableId', ),
			'without_payload':  True,
			'args':             args
		})




import unittest
import httpretty
from clayful.exception import ClayfulException
from clayful.requester import request

class ClayfulRequesterTest(unittest.TestCase):

	@httpretty.activate
	def test_request_with_query_and_headers(self):

		httpretty.register_uri(
			httpretty.GET,
			'https://api.clayful.io/v1/products',
			body='{ "success": true }',
			status=200,
			content_type='application/json'
		)

		result = request({
			'model_name':    'Product',
			'method_name':   'query',
			'http_method':   'GET',
			'request_url':   'https://api.clayful.io/v1/products',
			'query':         { 'raw': 'true' },
			'headers':       {
				'Authorization': 'Bearer client_token'
			},
			'payload':        None,
			'uses_form_data': False
		})

		last_request = httpretty.last_request()

		# Check query, headers
		self.assertEqual(last_request.path, '/v1/products?raw=true')
		self.assertEqual(last_request.headers.dict['authorization'], 'Bearer client_token')

		# Check the response
		self.assertEqual(result.status, 200)
		self.assertEqual(result.data['success'], True)
		self.assertEqual(result.headers['Content-Type'], 'application/json')


	@httpretty.activate
	def test_request_with_json_payload(self):

		httpretty.register_uri(
			httpretty.POST,
			'https://api.clayful.io/v1/products',
			body='{ "success": true }',
			status=201
		)

		result = request({
			'model_name':    'Product',
			'method_name':   'create',
			'http_method':   'POST',
			'request_url':   'https://api.clayful.io/v1/products',
			'query':         {},
			'headers':       {
				'Authorization': 'Bearer client_token'
			},
			'payload':        { 'name': {} },
			'uses_form_data': False
		})

		last_request = httpretty.last_request()

		# Check headers
		self.assertEqual(last_request.headers.dict['content-type'], 'application/json')

		self.assertEqual(result.status, 201)


	@httpretty.activate
	def test_request_with_multipart_form_data(self):

		httpretty.register_uri(
			httpretty.POST,
			'https://api.clayful.io/v1/images',
			body='{ "success": true }',
			status=201
		)

		result = request({
			'model_name':    'Image',
			'method_name':   'create',
			'http_method':   'POST',
			'request_url':   'https://api.clayful.io/v1/images',
			'query':         {},
			'headers':       {
				'Authorization': 'Bearer client_token'
			},
			'payload':        { 'file': 'file' },
			'uses_form_data': True
		})

		last_request = httpretty.last_request()

		# Check headers
		self.assertTrue('multipart/form-data' in last_request.headers.dict['content-type'])

		self.assertEqual(result.status, 201)


	@httpretty.activate
	def test_empty_response(self):

		httpretty.register_uri(
			httpretty.DELETE,
			'https://api.clayful.io/v1/products/abcdefg',
			status=204
		)

		result = request({
			'model_name':    'Product',
			'method_name':   'delete',
			'http_method':   'DELETE',
			'request_url':   'https://api.clayful.io/v1/products/abcdefg',
			'query':         {},
			'headers':       {
				'Authorization': 'Bearer client_token'
			},
			'payload':        None,
			'uses_form_data': False
		})

		self.assertEqual(result.status, 204)
		self.assertEqual(result.data, None)


	@httpretty.activate
	def test_clayful_error_case(self):

		httpretty.register_uri(
			httpretty.GET,
			'https://api.clayful.io/v1/products',
			body='{ "errorCode": "g-validation", "message": "validation error", "validation": {  } }',
			status=400,
			content_type='application/json'
		)

		try:

			request({
				'model_name':    'Product',
				'method_name':   'query',
				'http_method':   'GET',
				'request_url':   'https://api.clayful.io/v1/products',
				'query':         {},
				'headers':       {
					'Authorization': 'Bearer client_token'
				},
				'payload':        None,
				'uses_form_data': False
			})

		except ClayfulException as e:

			self.assertEqual(e.is_clayful, True)
			self.assertEqual(e.model, 'Product')
			self.assertEqual(e.method, 'query')
			self.assertEqual(e.status, 400)
			self.assertEqual(e.headers['Content-Type'], 'application/json')
			self.assertEqual(e.code, 'g-validation')
			self.assertEqual(e.message, 'validation error')
			self.assertEqual(e.validation, {})


class Image:

	Clayful = None
	name = 'Image'
	path = 'images'

	@staticmethod
	def config(clayful):

		Image.Clayful = clayful

		return Image

	@staticmethod
	def list(*args):

		return Image.Clayful.call_api({
			'model_name':       Image.name,
			'method_name':      'list',
			'http_method':      'GET',
			'path':             '/v1/images',
			'params':           (),
			'args':             args
		})

	@staticmethod
	def count(*args):

		return Image.Clayful.call_api({
			'model_name':       Image.name,
			'method_name':      'count',
			'http_method':      'GET',
			'path':             '/v1/images/count',
			'params':           (),
			'args':             args
		})

	@staticmethod
	def get(*args):

		return Image.Clayful.call_api({
			'model_name':       Image.name,
			'method_name':      'get',
			'http_method':      'GET',
			'path':             '/v1/images/{imageId}',
			'params':           ('imageId', ),
			'args':             args
		})

	@staticmethod
	def list_for_me(*args):

		return Image.Clayful.call_api({
			'model_name':       Image.name,
			'method_name':      'list_for_me',
			'http_method':      'GET',
			'path':             '/v1/me/images',
			'params':           (),
			'args':             args
		})

	@staticmethod
	def count_for_me(*args):

		return Image.Clayful.call_api({
			'model_name':       Image.name,
			'method_name':      'count_for_me',
			'http_method':      'GET',
			'path':             '/v1/me/images/count',
			'params':           (),
			'args':             args
		})

	@staticmethod
	def get_for_me(*args):

		return Image.Clayful.call_api({
			'model_name':       Image.name,
			'method_name':      'get_for_me',
			'http_method':      'GET',
			'path':             '/v1/me/images/{imageId}',
			'params':           ('imageId', ),
			'args':             args
		})

	@staticmethod
	def create(*args):

		return Image.Clayful.call_api({
			'model_name':       Image.name,
			'method_name':      'create',
			'http_method':      'POST',
			'path':             '/v1/images',
			'params':           (),
			'uses_form_data':   True,
			'args':             args
		})

	@staticmethod
	def create_for_me(*args):

		return Image.Clayful.call_api({
			'model_name':       Image.name,
			'method_name':      'create_for_me',
			'http_method':      'POST',
			'path':             '/v1/me/images',
			'params':           (),
			'uses_form_data':   True,
			'args':             args
		})

	@staticmethod
	def create_as_customer(*args):

		return Image.Clayful.call_api({
			'model_name':       Image.name,
			'method_name':      'create_as_customer',
			'http_method':      'POST',
			'path':             '/v1/customers/{customerId}/images',
			'params':           ('customerId', ),
			'uses_form_data':   True,
			'args':             args
		})

	@staticmethod
	def update(*args):

		return Image.Clayful.call_api({
			'model_name':       Image.name,
			'method_name':      'update',
			'http_method':      'PUT',
			'path':             '/v1/images/{imageId}',
			'params':           ('imageId', ),
			'uses_form_data':   True,
			'args':             args
		})

	@staticmethod
	def update_for_me(*args):

		return Image.Clayful.call_api({
			'model_name':       Image.name,
			'method_name':      'update_for_me',
			'http_method':      'PUT',
			'path':             '/v1/me/images/{imageId}',
			'params':           ('imageId', ),
			'uses_form_data':   True,
			'args':             args
		})

	@staticmethod
	def update_as_customer(*args):

		return Image.Clayful.call_api({
			'model_name':       Image.name,
			'method_name':      'update_as_customer',
			'http_method':      'PUT',
			'path':             '/v1/customers/{customerId}/images/{imageId}',
			'params':           ('customerId', 'imageId', ),
			'uses_form_data':   True,
			'args':             args
		})

	@staticmethod
	def delete(*args):

		return Image.Clayful.call_api({
			'model_name':       Image.name,
			'method_name':      'delete',
			'http_method':      'DELETE',
			'path':             '/v1/images/{imageId}',
			'params':           ('imageId', ),
			'args':             args
		})

	@staticmethod
	def delete_for_me(*args):

		return Image.Clayful.call_api({
			'model_name':       Image.name,
			'method_name':      'delete_for_me',
			'http_method':      'DELETE',
			'path':             '/v1/me/images/{imageId}',
			'params':           ('imageId', ),
			'args':             args
		})

	@staticmethod
	def delete_as_customer(*args):

		return Image.Clayful.call_api({
			'model_name':       Image.name,
			'method_name':      'delete_as_customer',
			'http_method':      'DELETE',
			'path':             '/v1/customers/{customerId}/images/{imageId}',
			'params':           ('customerId', 'imageId', ),
			'args':             args
		})




class ClayfulException(Exception):

	def __init__(
		self,
		model = None,
		method = None,
		status = None,
		headers = None,
		code = None,
		message = '',
		validation = None):

		self.is_clayful = True
		self.model = model
		self.method = method
		self.status = status
		self.headers = headers
		self.code = code
		self.message = message
		self.validation = validation

		super(ClayfulException, self).__init__(message)


class WishList:

	Clayful = None
	name = 'WishList'
	path = 'wishlists'

	@staticmethod
	def config(clayful):

		WishList.Clayful = clayful

		return WishList

	@staticmethod
	def list(*args):

		return WishList.Clayful.call_api({
			'model_name':       WishList.name,
			'method_name':      'list',
			'http_method':      'GET',
			'path':             '/v1/wishlists',
			'params':           (),
			'args':             args
		})

	@staticmethod
	def list_for_me(*args):

		return WishList.Clayful.call_api({
			'model_name':       WishList.name,
			'method_name':      'list_for_me',
			'http_method':      'GET',
			'path':             '/v1/me/wishlists',
			'params':           (),
			'args':             args
		})

	@staticmethod
	def count(*args):

		return WishList.Clayful.call_api({
			'model_name':       WishList.name,
			'method_name':      'count',
			'http_method':      'GET',
			'path':             '/v1/wishlists/count',
			'params':           (),
			'args':             args
		})

	@staticmethod
	def get(*args):

		return WishList.Clayful.call_api({
			'model_name':       WishList.name,
			'method_name':      'get',
			'http_method':      'GET',
			'path':             '/v1/wishlists/{wishListId}',
			'params':           ('wishListId', ),
			'args':             args
		})

	@staticmethod
	def count_for_me(*args):

		return WishList.Clayful.call_api({
			'model_name':       WishList.name,
			'method_name':      'count_for_me',
			'http_method':      'GET',
			'path':             '/v1/me/wishlists/count',
			'params':           (),
			'args':             args
		})

	@staticmethod
	def get_for_me(*args):

		return WishList.Clayful.call_api({
			'model_name':       WishList.name,
			'method_name':      'get_for_me',
			'http_method':      'GET',
			'path':             '/v1/me/wishlists/{wishListId}',
			'params':           ('wishListId', ),
			'args':             args
		})

	@staticmethod
	def list_products(*args):

		return WishList.Clayful.call_api({
			'model_name':       WishList.name,
			'method_name':      'list_products',
			'http_method':      'GET',
			'path':             '/v1/wishlists/{wishListId}/products',
			'params':           ('wishListId', ),
			'args':             args
		})

	@staticmethod
	def list_products_for_me(*args):

		return WishList.Clayful.call_api({
			'model_name':       WishList.name,
			'method_name':      'list_products_for_me',
			'http_method':      'GET',
			'path':             '/v1/me/wishlists/{wishListId}/products',
			'params':           ('wishListId', ),
			'args':             args
		})

	@staticmethod
	def count_products(*args):

		return WishList.Clayful.call_api({
			'model_name':       WishList.name,
			'method_name':      'count_products',
			'http_method':      'GET',
			'path':             '/v1/wishlists/{wishListId}/products/count',
			'params':           ('wishListId', ),
			'args':             args
		})

	@staticmethod
	def count_products_for_me(*args):

		return WishList.Clayful.call_api({
			'model_name':       WishList.name,
			'method_name':      'count_products_for_me',
			'http_method':      'GET',
			'path':             '/v1/me/wishlists/{wishListId}/products/count',
			'params':           ('wishListId', ),
			'args':             args
		})

	@staticmethod
	def create(*args):

		return WishList.Clayful.call_api({
			'model_name':       WishList.name,
			'method_name':      'create',
			'http_method':      'POST',
			'path':             '/v1/wishlists',
			'params':           (),
			'args':             args
		})

	@staticmethod
	def create_for_me(*args):

		return WishList.Clayful.call_api({
			'model_name':       WishList.name,
			'method_name':      'create_for_me',
			'http_method':      'POST',
			'path':             '/v1/me/wishlists',
			'params':           (),
			'args':             args
		})

	@staticmethod
	def add_item(*args):

		return WishList.Clayful.call_api({
			'model_name':       WishList.name,
			'method_name':      'add_item',
			'http_method':      'POST',
			'path':             '/v1/wishlists/{wishListId}/items',
			'params':           ('wishListId', ),
			'args':             args
		})

	@staticmethod
	def add_item_for_me(*args):

		return WishList.Clayful.call_api({
			'model_name':       WishList.name,
			'method_name':      'add_item_for_me',
			'http_method':      'POST',
			'path':             '/v1/me/wishlists/{wishListId}/items',
			'params':           ('wishListId', ),
			'args':             args
		})

	@staticmethod
	def increase_metafield(*args):

		return WishList.Clayful.call_api({
			'model_name':       WishList.name,
			'method_name':      'increase_metafield',
			'http_method':      'POST',
			'path':             '/v1/wishlists/{wishListId}/meta/{field}/inc',
			'params':           ('wishListId', 'field', ),
			'args':             args
		})

	@staticmethod
	def push_to_metafield(*args):

		return WishList.Clayful.call_api({
			'model_name':       WishList.name,
			'method_name':      'push_to_metafield',
			'http_method':      'POST',
			'path':             '/v1/wishlists/{wishListId}/meta/{field}/push',
			'params':           ('wishListId', 'field', ),
			'args':             args
		})

	@staticmethod
	def pull_from_metafield(*args):

		return WishList.Clayful.call_api({
			'model_name':       WishList.name,
			'method_name':      'pull_from_metafield',
			'http_method':      'POST',
			'path':             '/v1/wishlists/{wishListId}/meta/{field}/pull',
			'params':           ('wishListId', 'field', ),
			'args':             args
		})

	@staticmethod
	def update(*args):

		return WishList.Clayful.call_api({
			'model_name':       WishList.name,
			'method_name':      'update',
			'http_method':      'PUT',
			'path':             '/v1/wishlists/{wishListId}',
			'params':           ('wishListId', ),
			'args':             args
		})

	@staticmethod
	def update_for_me(*args):

		return WishList.Clayful.call_api({
			'model_name':       WishList.name,
			'method_name':      'update_for_me',
			'http_method':      'PUT',
			'path':             '/v1/me/wishlists/{wishListId}',
			'params':           ('wishListId', ),
			'args':             args
		})

	@staticmethod
	def delete(*args):

		return WishList.Clayful.call_api({
			'model_name':       WishList.name,
			'method_name':      'delete',
			'http_method':      'DELETE',
			'path':             '/v1/wishlists/{wishListId}',
			'params':           ('wishListId', ),
			'args':             args
		})

	@staticmethod
	def delete_for_me(*args):

		return WishList.Clayful.call_api({
			'model_name':       WishList.name,
			'method_name':      'delete_for_me',
			'http_method':      'DELETE',
			'path':             '/v1/me/wishlists/{wishListId}',
			'params':           ('wishListId', ),
			'args':             args
		})

	@staticmethod
	def empty(*args):

		return WishList.Clayful.call_api({
			'model_name':       WishList.name,
			'method_name':      'empty',
			'http_method':      'DELETE',
			'path':             '/v1/wishlists/{wishListId}/items',
			'params':           ('wishListId', ),
			'args':             args
		})

	@staticmethod
	def empty_for_me(*args):

		return WishList.Clayful.call_api({
			'model_name':       WishList.name,
			'method_name':      'empty_for_me',
			'http_method':      'DELETE',
			'path':             '/v1/me/wishlists/{wishListId}/items',
			'params':           ('wishListId', ),
			'args':             args
		})

	@staticmethod
	def delete_item(*args):

		return WishList.Clayful.call_api({
			'model_name':       WishList.name,
			'method_name':      'delete_item',
			'http_method':      'DELETE',
			'path':             '/v1/wishlists/{wishListId}/items/{productId}',
			'params':           ('wishListId', 'productId', ),
			'args':             args
		})

	@staticmethod
	def delete_metafield(*args):

		return WishList.Clayful.call_api({
			'model_name':       WishList.name,
			'method_name':      'delete_metafield',
			'http_method':      'DELETE',
			'path':             '/v1/wishlists/{wishListId}/meta/{field}',
			'params':           ('wishListId', 'field', ),
			'args':             args
		})

	@staticmethod
	def delete_item_for_me(*args):

		return WishList.Clayful.call_api({
			'model_name':       WishList.name,
			'method_name':      'delete_item_for_me',
			'http_method':      'DELETE',
			'path':             '/v1/me/wishlists/{wishListId}/items/{productId}',
			'params':           ('wishListId', 'productId', ),
			'args':             args
		})




import re
import urllib
import numbers
from clayful.models import register_models
from clayful.requester import request
from clayful.exception import ClayfulException

class Clayful:

	base_url = 'https://api.clayful.io'

	default_headers = {
		'Accept-Encoding': 'gzip',
		'User-Agent':      'clayful-python',
		'Clayful-SDK':     'clayful-python'
	}

	plugins = {
		'request': request
	}

	listeners = {
		'request':  [],
		'response': []
	}

	@staticmethod
	def options_to_headers(o = {}):

		headers = {}

		if 'language' in o:
			headers['Accept-Language'] = o['language']

		if 'currency' in o:
			headers['Accept-Currency'] = o['currency']

		if 'time_zone' in o:
			headers['Accept-Time-Zone'] = o['time_zone']

		if 'client' in o:
			headers['Authorization'] = 'Bearer ' + o['client']

		if 'customer' in o:
			headers['Authorization-Customer'] = o['customer']

		if 'reCAPTCHA' in o:
			headers['reCAPTCHA-Response'] = o['reCAPTCHA'];

		if 'debug_language' in o:
			headers['Accept-Debug-Language'] = o['debug_language']

		if 'headers' in o:
			headers.update(o['headers'])

		return headers


	@staticmethod
	def get_endpoint(path):

		return Clayful.base_url + path

	@staticmethod
	def normalize_query_values(query = {}):

		copied = query.copy()

		for key in copied:

			if isinstance(copied[key], bool):

				copied[key] = 'true' if copied[key] == True else 'false'

			if isinstance(copied[key], numbers.Number):

				copied[key] = str(copied[key])

		return copied


	@staticmethod
	def extract_request_arguments(options):

		result = {
			'http_method': options['http_method'],
			'request_url': options['path'],
			'payload':     None,
			'query':       {},
			'headers':     {},
			'meta':        {}
		}

		rest = options['args'][len(options['params']):]

		for i, key in enumerate(options['params']):

			result['request_url'] = result['request_url'].replace('{' + key + '}', str(options['args'][i]))

		if (options['http_method'] == 'POST' or options['http_method'] == 'PUT') and (options.get('without_payload', False) == False):

			result['payload'] = (rest[0:1] or (None,))[0]
			rest = rest[1:]

		query_headers = (rest[0:1] or ({},))[0]

		result['query'] = Clayful.normalize_query_values(query_headers.get('query', {}))
		result['headers'] = Clayful.options_to_headers(query_headers)
		result['meta'] = query_headers.get('meta', {})

		return result


	@staticmethod
	def call_api(options):

		extracted = Clayful.extract_request_arguments(options)
		extracted.update({
			'request_url':    Clayful.get_endpoint(extracted['request_url']),
			'model_name':     options['model_name'],
			'method_name':    options['method_name'],
			'uses_form_data': options.get('uses_form_data', False),
			'error':          None,
			'response':       None,
		})

		default_headers = Clayful.default_headers.copy()

		# Extend default headers with header options
		default_headers.update(extracted['headers'])

		extracted['headers'] = default_headers

		Clayful.trigger('request', extracted)

		try:

			response = Clayful.plugins['request'](extracted)

			extracted['response'] = response

			Clayful.trigger('response', extracted)

			return response

		except ClayfulException as e:

			extracted['error'] = e

			Clayful.trigger('response', extracted)

			raise


	@staticmethod
	def config(options = {}):

		headers = Clayful.options_to_headers(options)

		Clayful.default_headers.update(headers)


	@staticmethod
	def install(scope, plugin):

		if scope in Clayful.plugins:

			Clayful.plugins[scope] = plugin


	@staticmethod
	def on(event_name, callback):

		listeners = Clayful.listeners.get(event_name, None)

		if listeners is None:
			return

		listeners.append(callback)


	@staticmethod
	def off(event_name, callback):

		listeners = Clayful.listeners.get(event_name, None)

		if (listeners is None) or (not callback in listeners):
			return

		listeners.remove(callback)


	@staticmethod
	def trigger(event_name, data):

		listeners = Clayful.listeners.get(event_name, None)

		if listeners is None:
			return

		for listener in listeners:
			listener(data)


	@staticmethod
	def format_image_url(base_url, options = {}):

		query = []

		normalized = Clayful.normalize_query_values(options)

		for key in normalized:

			query.append(key + '=' + normalized.get(key, ''))

		query = '&'.join(query)

		if bool(query):

			query = '?' + query

		return base_url + query


	@staticmethod
	def format_number(number, currency = {}):

		if not isinstance(number, numbers.Number):

			return ''

		precision = currency.get('precision', None)
		delimiter = currency.get('delimiter', {})
		thousands = delimiter.get('thousands', '')
		decimal = delimiter.get('decimal', '.')

		if isinstance(precision, numbers.Number):

			n = 10 ** precision
			number = round(number * n) / n

			# To deal with 0.0 case..
			if precision == 0:
				number = int(number)

		parts = str(number).split('.')


		a = thousands.join(re.findall('.{1,3}', parts[0][::-1]))[::-1]
		b = parts[1] if len(parts) > 1 else ''

		if isinstance(precision, numbers.Number):

			diff = precision - len(b)
			diff = 0 if diff < 0 else diff

			b += '0' * diff

		decimal = decimal if bool(b) else ''

		return decimal.join([a, b])


	@staticmethod
	def format_price(number, currency = {}):

		formatted_number = Clayful.format_number(number, currency)

		if not bool(formatted_number):

			return ''

		symbol = currency.get('symbol', '')
		format = currency.get('format', '{price}')

		return format.replace('{symbol}', symbol).replace('{price}', formatted_number)


# Register models
register_models(Clayful)


class Vendor:

	Clayful = None
	name = 'Vendor'
	path = 'vendors'

	@staticmethod
	def config(clayful):

		Vendor.Clayful = clayful

		return Vendor

	@staticmethod
	def list(*args):

		return Vendor.Clayful.call_api({
			'model_name':       Vendor.name,
			'method_name':      'list',
			'http_method':      'GET',
			'path':             '/v1/vendors',
			'params':           (),
			'args':             args
		})

	@staticmethod
	def count(*args):

		return Vendor.Clayful.call_api({
			'model_name':       Vendor.name,
			'method_name':      'count',
			'http_method':      'GET',
			'path':             '/v1/vendors/count',
			'params':           (),
			'args':             args
		})

	@staticmethod
	def get(*args):

		return Vendor.Clayful.call_api({
			'model_name':       Vendor.name,
			'method_name':      'get',
			'http_method':      'GET',
			'path':             '/v1/vendors/{vendorId}',
			'params':           ('vendorId', ),
			'args':             args
		})

	@staticmethod
	def create(*args):

		return Vendor.Clayful.call_api({
			'model_name':       Vendor.name,
			'method_name':      'create',
			'http_method':      'POST',
			'path':             '/v1/vendors',
			'params':           (),
			'args':             args
		})

	@staticmethod
	def push_to_metafield(*args):

		return Vendor.Clayful.call_api({
			'model_name':       Vendor.name,
			'method_name':      'push_to_metafield',
			'http_method':      'POST',
			'path':             '/v1/vendors/{vendorId}/meta/{field}/push',
			'params':           ('vendorId', 'field', ),
			'args':             args
		})

	@staticmethod
	def increase_metafield(*args):

		return Vendor.Clayful.call_api({
			'model_name':       Vendor.name,
			'method_name':      'increase_metafield',
			'http_method':      'POST',
			'path':             '/v1/vendors/{vendorId}/meta/{field}/inc',
			'params':           ('vendorId', 'field', ),
			'args':             args
		})

	@staticmethod
	def pull_from_metafield(*args):

		return Vendor.Clayful.call_api({
			'model_name':       Vendor.name,
			'method_name':      'pull_from_metafield',
			'http_method':      'POST',
			'path':             '/v1/vendors/{vendorId}/meta/{field}/pull',
			'params':           ('vendorId', 'field', ),
			'args':             args
		})

	@staticmethod
	def update(*args):

		return Vendor.Clayful.call_api({
			'model_name':       Vendor.name,
			'method_name':      'update',
			'http_method':      'PUT',
			'path':             '/v1/vendors/{vendorId}',
			'params':           ('vendorId', ),
			'args':             args
		})

	@staticmethod
	def delete(*args):

		return Vendor.Clayful.call_api({
			'model_name':       Vendor.name,
			'method_name':      'delete',
			'http_method':      'DELETE',
			'path':             '/v1/vendors/{vendorId}',
			'params':           ('vendorId', ),
			'args':             args
		})

	@staticmethod
	def delete_metafield(*args):

		return Vendor.Clayful.call_api({
			'model_name':       Vendor.name,
			'method_name':      'delete_metafield',
			'http_method':      'DELETE',
			'path':             '/v1/vendors/{vendorId}/meta/{field}',
			'params':           ('vendorId', 'field', ),
			'args':             args
		})




from clayful.models.brand import Brand
from clayful.models.cart import Cart
from clayful.models.catalog import Catalog
from clayful.models.collection import Collection
from clayful.models.country import Country
from clayful.models.coupon import Coupon
from clayful.models.currency import Currency
from clayful.models.customer import Customer
from clayful.models.discount import Discount
from clayful.models.downloadable import Downloadable
from clayful.models.group import Group
from clayful.models.image import Image
from clayful.models.order import Order
from clayful.models.order_tag import OrderTag
from clayful.models.payment_method import PaymentMethod
from clayful.models.product import Product
from clayful.models.review import Review
from clayful.models.review_comment import ReviewComment
from clayful.models.shipping_method import ShippingMethod
from clayful.models.store import Store
from clayful.models.subscription import Subscription
from clayful.models.subscription_plan import SubscriptionPlan
from clayful.models.tax_category import TaxCategory
from clayful.models.vendor import Vendor
from clayful.models.wish_list import WishList

def register_models(clayful):

	setattr(clayful, 'Brand', Brand.config(clayful))
	setattr(clayful, 'Cart', Cart.config(clayful))
	setattr(clayful, 'Catalog', Catalog.config(clayful))
	setattr(clayful, 'Collection', Collection.config(clayful))
	setattr(clayful, 'Country', Country.config(clayful))
	setattr(clayful, 'Coupon', Coupon.config(clayful))
	setattr(clayful, 'Currency', Currency.config(clayful))
	setattr(clayful, 'Customer', Customer.config(clayful))
	setattr(clayful, 'Discount', Discount.config(clayful))
	setattr(clayful, 'Downloadable', Downloadable.config(clayful))
	setattr(clayful, 'Group', Group.config(clayful))
	setattr(clayful, 'Image', Image.config(clayful))
	setattr(clayful, 'Order', Order.config(clayful))
	setattr(clayful, 'OrderTag', OrderTag.config(clayful))
	setattr(clayful, 'PaymentMethod', PaymentMethod.config(clayful))
	setattr(clayful, 'Product', Product.config(clayful))
	setattr(clayful, 'Review', Review.config(clayful))
	setattr(clayful, 'ReviewComment', ReviewComment.config(clayful))
	setattr(clayful, 'ShippingMethod', ShippingMethod.config(clayful))
	setattr(clayful, 'Store', Store.config(clayful))
	setattr(clayful, 'Subscription', Subscription.config(clayful))
	setattr(clayful, 'SubscriptionPlan', SubscriptionPlan.config(clayful))
	setattr(clayful, 'TaxCategory', TaxCategory.config(clayful))
	setattr(clayful, 'Vendor', Vendor.config(clayful))
	setattr(clayful, 'WishList', WishList.config(clayful))



class TaxCategory:

	Clayful = None
	name = 'TaxCategory'
	path = 'taxes/categories'

	@staticmethod
	def config(clayful):

		TaxCategory.Clayful = clayful

		return TaxCategory

	@staticmethod
	def list(*args):

		return TaxCategory.Clayful.call_api({
			'model_name':       TaxCategory.name,
			'method_name':      'list',
			'http_method':      'GET',
			'path':             '/v1/taxes/categories',
			'params':           (),
			'args':             args
		})

	@staticmethod
	def count(*args):

		return TaxCategory.Clayful.call_api({
			'model_name':       TaxCategory.name,
			'method_name':      'count',
			'http_method':      'GET',
			'path':             '/v1/taxes/categories/count',
			'params':           (),
			'args':             args
		})

	@staticmethod
	def get(*args):

		return TaxCategory.Clayful.call_api({
			'model_name':       TaxCategory.name,
			'method_name':      'get',
			'http_method':      'GET',
			'path':             '/v1/taxes/categories/{taxCategoryId}',
			'params':           ('taxCategoryId', ),
			'args':             args
		})




class Order:

	Clayful = None
	name = 'Order'
	path = 'orders'

	@staticmethod
	def config(clayful):

		Order.Clayful = clayful

		return Order

	@staticmethod
	def list(*args):

		return Order.Clayful.call_api({
			'model_name':       Order.name,
			'method_name':      'list',
			'http_method':      'GET',
			'path':             '/v1/orders',
			'params':           (),
			'args':             args
		})

	@staticmethod
	def list_for_me(*args):

		return Order.Clayful.call_api({
			'model_name':       Order.name,
			'method_name':      'list_for_me',
			'http_method':      'GET',
			'path':             '/v1/me/orders',
			'params':           (),
			'args':             args
		})

	@staticmethod
	def count(*args):

		return Order.Clayful.call_api({
			'model_name':       Order.name,
			'method_name':      'count',
			'http_method':      'GET',
			'path':             '/v1/orders/count',
			'params':           (),
			'args':             args
		})

	@staticmethod
	def get(*args):

		return Order.Clayful.call_api({
			'model_name':       Order.name,
			'method_name':      'get',
			'http_method':      'GET',
			'path':             '/v1/orders/{orderId}',
			'params':           ('orderId', ),
			'args':             args
		})

	@staticmethod
	def count_for_me(*args):

		return Order.Clayful.call_api({
			'model_name':       Order.name,
			'method_name':      'count_for_me',
			'http_method':      'GET',
			'path':             '/v1/me/orders/count',
			'params':           (),
			'args':             args
		})

	@staticmethod
	def get_for_me(*args):

		return Order.Clayful.call_api({
			'model_name':       Order.name,
			'method_name':      'get_for_me',
			'http_method':      'GET',
			'path':             '/v1/me/orders/{orderId}',
			'params':           ('orderId', ),
			'args':             args
		})

	@staticmethod
	def list_by_subscription(*args):

		return Order.Clayful.call_api({
			'model_name':       Order.name,
			'method_name':      'list_by_subscription',
			'http_method':      'GET',
			'path':             '/v1/subscriptions/{subscriptionId}/orders',
			'params':           ('subscriptionId', ),
			'args':             args
		})

	@staticmethod
	def list_by_subscription_for_me(*args):

		return Order.Clayful.call_api({
			'model_name':       Order.name,
			'method_name':      'list_by_subscription_for_me',
			'http_method':      'GET',
			'path':             '/v1/me/subscriptions/{subscriptionId}/orders',
			'params':           ('subscriptionId', ),
			'args':             args
		})

	@staticmethod
	def list_inventory_operations(*args):

		return Order.Clayful.call_api({
			'model_name':       Order.name,
			'method_name':      'list_inventory_operations',
			'http_method':      'GET',
			'path':             '/v1/orders/{orderId}/inventory/operations',
			'params':           ('orderId', ),
			'args':             args
		})

	@staticmethod
	def mark_as_done(*args):

		return Order.Clayful.call_api({
			'model_name':       Order.name,
			'method_name':      'mark_as_done',
			'http_method':      'POST',
			'path':             '/v1/orders/{orderId}/done',
			'params':           ('orderId', ),
			'without_payload':  True,
			'args':             args
		})

	@staticmethod
	def sync_inventory(*args):

		return Order.Clayful.call_api({
			'model_name':       Order.name,
			'method_name':      'sync_inventory',
			'http_method':      'POST',
			'path':             '/v1/orders/{orderId}/synced',
			'params':           ('orderId', ),
			'without_payload':  True,
			'args':             args
		})

	@staticmethod
	def authenticate(*args):

		return Order.Clayful.call_api({
			'model_name':       Order.name,
			'method_name':      'authenticate',
			'http_method':      'POST',
			'path':             '/v1/orders/{orderId}/auth',
			'params':           ('orderId', ),
			'args':             args
		})

	@staticmethod
	def cancel(*args):

		return Order.Clayful.call_api({
			'model_name':       Order.name,
			'method_name':      'cancel',
			'http_method':      'POST',
			'path':             '/v1/orders/{orderId}/cancellation',
			'params':           ('orderId', ),
			'args':             args
		})

	@staticmethod
	def request_refund(*args):

		return Order.Clayful.call_api({
			'model_name':       Order.name,
			'method_name':      'request_refund',
			'http_method':      'POST',
			'path':             '/v1/orders/{orderId}/refunds',
			'params':           ('orderId', ),
			'args':             args
		})

	@staticmethod
	def create_fulfillment(*args):

		return Order.Clayful.call_api({
			'model_name':       Order.name,
			'method_name':      'create_fulfillment',
			'http_method':      'POST',
			'path':             '/v1/orders/{orderId}/fulfillments',
			'params':           ('orderId', ),
			'args':             args
		})

	@staticmethod
	def mark_as_received(*args):

		return Order.Clayful.call_api({
			'model_name':       Order.name,
			'method_name':      'mark_as_received',
			'http_method':      'POST',
			'path':             '/v1/orders/{orderId}/received',
			'params':           ('orderId', ),
			'without_payload':  True,
			'args':             args
		})

	@staticmethod
	def cancel_for_me(*args):

		return Order.Clayful.call_api({
			'model_name':       Order.name,
			'method_name':      'cancel_for_me',
			'http_method':      'POST',
			'path':             '/v1/me/orders/{orderId}/cancellation',
			'params':           ('orderId', ),
			'args':             args
		})

	@staticmethod
	def mark_as_received_for_me(*args):

		return Order.Clayful.call_api({
			'model_name':       Order.name,
			'method_name':      'mark_as_received_for_me',
			'http_method':      'POST',
			'path':             '/v1/me/orders/{orderId}/received',
			'params':           ('orderId', ),
			'without_payload':  True,
			'args':             args
		})

	@staticmethod
	def request_refund_for_me(*args):

		return Order.Clayful.call_api({
			'model_name':       Order.name,
			'method_name':      'request_refund_for_me',
			'http_method':      'POST',
			'path':             '/v1/me/orders/{orderId}/refunds',
			'params':           ('orderId', ),
			'args':             args
		})

	@staticmethod
	def check_ticket(*args):

		return Order.Clayful.call_api({
			'model_name':       Order.name,
			'method_name':      'check_ticket',
			'http_method':      'POST',
			'path':             '/v1/orders/tickets/{code}/validity',
			'params':           ('code', ),
			'args':             args
		})

	@staticmethod
	def use_ticket(*args):

		return Order.Clayful.call_api({
			'model_name':       Order.name,
			'method_name':      'use_ticket',
			'http_method':      'POST',
			'path':             '/v1/orders/tickets/{code}/used',
			'params':           ('code', ),
			'without_payload':  True,
			'args':             args
		})

	@staticmethod
	def increase_metafield(*args):

		return Order.Clayful.call_api({
			'model_name':       Order.name,
			'method_name':      'increase_metafield',
			'http_method':      'POST',
			'path':             '/v1/orders/{orderId}/meta/{field}/inc',
			'params':           ('orderId', 'field', ),
			'args':             args
		})

	@staticmethod
	def restock_refund_items(*args):

		return Order.Clayful.call_api({
			'model_name':       Order.name,
			'method_name':      'restock_refund_items',
			'http_method':      'POST',
			'path':             '/v1/orders/{orderId}/refunds/{refundId}/restock',
			'params':           ('orderId', 'refundId', ),
			'args':             args
		})

	@staticmethod
	def pull_from_metafield(*args):

		return Order.Clayful.call_api({
			'model_name':       Order.name,
			'method_name':      'pull_from_metafield',
			'http_method':      'POST',
			'path':             '/v1/orders/{orderId}/meta/{field}/pull',
			'params':           ('orderId', 'field', ),
			'args':             args
		})

	@staticmethod
	def accept_refund(*args):

		return Order.Clayful.call_api({
			'model_name':       Order.name,
			'method_name':      'accept_refund',
			'http_method':      'POST',
			'path':             '/v1/orders/{orderId}/refunds/{refundId}/accepted',
			'params':           ('orderId', 'refundId', ),
			'without_payload':  True,
			'args':             args
		})

	@staticmethod
	def cancel_refund(*args):

		return Order.Clayful.call_api({
			'model_name':       Order.name,
			'method_name':      'cancel_refund',
			'http_method':      'POST',
			'path':             '/v1/orders/{orderId}/refunds/{refundId}/cancellation',
			'params':           ('orderId', 'refundId', ),
			'args':             args
		})

	@staticmethod
	def push_to_metafield(*args):

		return Order.Clayful.call_api({
			'model_name':       Order.name,
			'method_name':      'push_to_metafield',
			'http_method':      'POST',
			'path':             '/v1/orders/{orderId}/meta/{field}/push',
			'params':           ('orderId', 'field', ),
			'args':             args
		})

	@staticmethod
	def register_payment_method(*args):

		return Order.Clayful.call_api({
			'model_name':       Order.name,
			'method_name':      'register_payment_method',
			'http_method':      'POST',
			'path':             '/v1/orders/{orderId}/transactions/payments/methods',
			'params':           ('orderId', ),
			'args':             args
		})

	@staticmethod
	def cancel_refund_for_me(*args):

		return Order.Clayful.call_api({
			'model_name':       Order.name,
			'method_name':      'cancel_refund_for_me',
			'http_method':      'POST',
			'path':             '/v1/me/orders/{orderId}/refunds/{refundId}/cancellation',
			'params':           ('orderId', 'refundId', ),
			'args':             args
		})

	@staticmethod
	def create_download_url(*args):

		return Order.Clayful.call_api({
			'model_name':       Order.name,
			'method_name':      'create_download_url',
			'http_method':      'POST',
			'path':             '/v1/orders/{orderId}/items/{itemId}/download/url',
			'params':           ('orderId', 'itemId', ),
			'without_payload':  True,
			'args':             args
		})

	@staticmethod
	def restock_all_refund_items(*args):

		return Order.Clayful.call_api({
			'model_name':       Order.name,
			'method_name':      'restock_all_refund_items',
			'http_method':      'POST',
			'path':             '/v1/orders/{orderId}/refunds/{refundId}/restock/all',
			'params':           ('orderId', 'refundId', ),
			'without_payload':  True,
			'args':             args
		})

	@staticmethod
	def create_download_url_for_me(*args):

		return Order.Clayful.call_api({
			'model_name':       Order.name,
			'method_name':      'create_download_url_for_me',
			'http_method':      'POST',
			'path':             '/v1/me/orders/{orderId}/items/{itemId}/download/url',
			'params':           ('orderId', 'itemId', ),
			'without_payload':  True,
			'args':             args
		})

	@staticmethod
	def update(*args):

		return Order.Clayful.call_api({
			'model_name':       Order.name,
			'method_name':      'update',
			'http_method':      'PUT',
			'path':             '/v1/orders/{orderId}',
			'params':           ('orderId', ),
			'args':             args
		})

	@staticmethod
	def update_for_me(*args):

		return Order.Clayful.call_api({
			'model_name':       Order.name,
			'method_name':      'update_for_me',
			'http_method':      'PUT',
			'path':             '/v1/me/orders/{orderId}',
			'params':           ('orderId', ),
			'args':             args
		})

	@staticmethod
	def update_cancellation(*args):

		return Order.Clayful.call_api({
			'model_name':       Order.name,
			'method_name':      'update_cancellation',
			'http_method':      'PUT',
			'path':             '/v1/orders/{orderId}/cancellation',
			'params':           ('orderId', ),
			'args':             args
		})

	@staticmethod
	def update_transactions(*args):

		return Order.Clayful.call_api({
			'model_name':       Order.name,
			'method_name':      'update_transactions',
			'http_method':      'PUT',
			'path':             '/v1/orders/{orderId}/transactions',
			'params':           ('orderId', ),
			'args':             args
		})

	@staticmethod
	def update_transactions_for_me(*args):

		return Order.Clayful.call_api({
			'model_name':       Order.name,
			'method_name':      'update_transactions_for_me',
			'http_method':      'PUT',
			'path':             '/v1/me/orders/{orderId}/transactions',
			'params':           ('orderId', ),
			'without_payload':  True,
			'args':             args
		})

	@staticmethod
	def update_cancellation_for_me(*args):

		return Order.Clayful.call_api({
			'model_name':       Order.name,
			'method_name':      'update_cancellation_for_me',
			'http_method':      'PUT',
			'path':             '/v1/me/orders/{orderId}/cancellation',
			'params':           ('orderId', ),
			'args':             args
		})

	@staticmethod
	def update_item(*args):

		return Order.Clayful.call_api({
			'model_name':       Order.name,
			'method_name':      'update_item',
			'http_method':      'PUT',
			'path':             '/v1/orders/{orderId}/items/{itemId}',
			'params':           ('orderId', 'itemId', ),
			'args':             args
		})

	@staticmethod
	def update_refund(*args):

		return Order.Clayful.call_api({
			'model_name':       Order.name,
			'method_name':      'update_refund',
			'http_method':      'PUT',
			'path':             '/v1/orders/{orderId}/refunds/{refundId}',
			'params':           ('orderId', 'refundId', ),
			'args':             args
		})

	@staticmethod
	def update_fulfillment(*args):

		return Order.Clayful.call_api({
			'model_name':       Order.name,
			'method_name':      'update_fulfillment',
			'http_method':      'PUT',
			'path':             '/v1/orders/{orderId}/fulfillments/{fulfillmentId}',
			'params':           ('orderId', 'fulfillmentId', ),
			'args':             args
		})

	@staticmethod
	def update_refund_for_me(*args):

		return Order.Clayful.call_api({
			'model_name':       Order.name,
			'method_name':      'update_refund_for_me',
			'http_method':      'PUT',
			'path':             '/v1/me/orders/{orderId}/refunds/{refundId}',
			'params':           ('orderId', 'refundId', ),
			'args':             args
		})

	@staticmethod
	def update_refund_cancellation(*args):

		return Order.Clayful.call_api({
			'model_name':       Order.name,
			'method_name':      'update_refund_cancellation',
			'http_method':      'PUT',
			'path':             '/v1/orders/{orderId}/refunds/{refundId}/cancellation',
			'params':           ('orderId', 'refundId', ),
			'args':             args
		})

	@staticmethod
	def update_refund_cancellation_for_me(*args):

		return Order.Clayful.call_api({
			'model_name':       Order.name,
			'method_name':      'update_refund_cancellation_for_me',
			'http_method':      'PUT',
			'path':             '/v1/me/orders/{orderId}/refunds/{refundId}/cancellation',
			'params':           ('orderId', 'refundId', ),
			'args':             args
		})

	@staticmethod
	def delete(*args):

		return Order.Clayful.call_api({
			'model_name':       Order.name,
			'method_name':      'delete',
			'http_method':      'DELETE',
			'path':             '/v1/orders/{orderId}',
			'params':           ('orderId', ),
			'args':             args
		})

	@staticmethod
	def mark_as_not_received(*args):

		return Order.Clayful.call_api({
			'model_name':       Order.name,
			'method_name':      'mark_as_not_received',
			'http_method':      'DELETE',
			'path':             '/v1/orders/{orderId}/received',
			'params':           ('orderId', ),
			'args':             args
		})

	@staticmethod
	def mark_as_undone(*args):

		return Order.Clayful.call_api({
			'model_name':       Order.name,
			'method_name':      'mark_as_undone',
			'http_method':      'DELETE',
			'path':             '/v1/orders/{orderId}/done',
			'params':           ('orderId', ),
			'args':             args
		})

	@staticmethod
	def mark_as_not_received_for_me(*args):

		return Order.Clayful.call_api({
			'model_name':       Order.name,
			'method_name':      'mark_as_not_received_for_me',
			'http_method':      'DELETE',
			'path':             '/v1/me/orders/{orderId}/received',
			'params':           ('orderId', ),
			'args':             args
		})

	@staticmethod
	def delete_fulfillment(*args):

		return Order.Clayful.call_api({
			'model_name':       Order.name,
			'method_name':      'delete_fulfillment',
			'http_method':      'DELETE',
			'path':             '/v1/orders/{orderId}/fulfillments/{fulfillmentId}',
			'params':           ('orderId', 'fulfillmentId', ),
			'args':             args
		})

	@staticmethod
	def delete_metafield(*args):

		return Order.Clayful.call_api({
			'model_name':       Order.name,
			'method_name':      'delete_metafield',
			'http_method':      'DELETE',
			'path':             '/v1/orders/{orderId}/meta/{field}',
			'params':           ('orderId', 'field', ),
			'args':             args
		})

	@staticmethod
	def delete_refund(*args):

		return Order.Clayful.call_api({
			'model_name':       Order.name,
			'method_name':      'delete_refund',
			'http_method':      'DELETE',
			'path':             '/v1/orders/{orderId}/refunds/{refundId}',
			'params':           ('orderId', 'refundId', ),
			'args':             args
		})

	@staticmethod
	def delete_inventory_operation(*args):

		return Order.Clayful.call_api({
			'model_name':       Order.name,
			'method_name':      'delete_inventory_operation',
			'http_method':      'DELETE',
			'path':             '/v1/orders/{orderId}/inventory/operations/{operationId}',
			'params':           ('orderId', 'operationId', ),
			'args':             args
		})

	@staticmethod
	def unaccept_refund(*args):

		return Order.Clayful.call_api({
			'model_name':       Order.name,
			'method_name':      'unaccept_refund',
			'http_method':      'DELETE',
			'path':             '/v1/orders/{orderId}/refunds/{refundId}/accepted',
			'params':           ('orderId', 'refundId', ),
			'args':             args
		})

	@staticmethod
	def unregister_payment_method(*args):

		return Order.Clayful.call_api({
			'model_name':       Order.name,
			'method_name':      'unregister_payment_method',
			'http_method':      'DELETE',
			'path':             '/v1/orders/{orderId}/transactions/payments/methods/{paymentMethodId}',
			'params':           ('orderId', 'paymentMethodId', ),
			'args':             args
		})




class Customer:

	Clayful = None
	name = 'Customer'
	path = 'customers'

	@staticmethod
	def config(clayful):

		Customer.Clayful = clayful

		return Customer

	@staticmethod
	def list(*args):

		return Customer.Clayful.call_api({
			'model_name':       Customer.name,
			'method_name':      'list',
			'http_method':      'GET',
			'path':             '/v1/customers',
			'params':           (),
			'args':             args
		})

	@staticmethod
	def get_me(*args):

		return Customer.Clayful.call_api({
			'model_name':       Customer.name,
			'method_name':      'get_me',
			'http_method':      'GET',
			'path':             '/v1/me',
			'params':           (),
			'args':             args
		})

	@staticmethod
	def is_authenticated(*args):

		return Customer.Clayful.call_api({
			'model_name':       Customer.name,
			'method_name':      'is_authenticated',
			'http_method':      'GET',
			'path':             '/v1/customers/auth',
			'params':           (),
			'args':             args
		})

	@staticmethod
	def count(*args):

		return Customer.Clayful.call_api({
			'model_name':       Customer.name,
			'method_name':      'count',
			'http_method':      'GET',
			'path':             '/v1/customers/count',
			'params':           (),
			'args':             args
		})

	@staticmethod
	def get(*args):

		return Customer.Clayful.call_api({
			'model_name':       Customer.name,
			'method_name':      'get',
			'http_method':      'GET',
			'path':             '/v1/customers/{customerId}',
			'params':           ('customerId', ),
			'args':             args
		})

	@staticmethod
	def list_coupons_for_me(*args):

		return Customer.Clayful.call_api({
			'model_name':       Customer.name,
			'method_name':      'list_coupons_for_me',
			'http_method':      'GET',
			'path':             '/v1/me/coupons',
			'params':           (),
			'args':             args
		})

	@staticmethod
	def list_coupons(*args):

		return Customer.Clayful.call_api({
			'model_name':       Customer.name,
			'method_name':      'list_coupons',
			'http_method':      'GET',
			'path':             '/v1/customers/{customerId}/coupons',
			'params':           ('customerId', ),
			'args':             args
		})

	@staticmethod
	def count_coupons_for_me(*args):

		return Customer.Clayful.call_api({
			'model_name':       Customer.name,
			'method_name':      'count_coupons_for_me',
			'http_method':      'GET',
			'path':             '/v1/me/coupons/count',
			'params':           (),
			'args':             args
		})

	@staticmethod
	def count_coupons(*args):

		return Customer.Clayful.call_api({
			'model_name':       Customer.name,
			'method_name':      'count_coupons',
			'http_method':      'GET',
			'path':             '/v1/customers/{customerId}/coupons/count',
			'params':           ('customerId', ),
			'args':             args
		})

	@staticmethod
	def list_by_flag_votes(*args):

		return Customer.Clayful.call_api({
			'model_name':       Customer.name,
			'method_name':      'list_by_flag_votes',
			'http_method':      'GET',
			'path':             '/v1/{voteModel}/{voteModelId}/flags/customers',
			'params':           ('voteModel', 'voteModelId', ),
			'args':             args
		})

	@staticmethod
	def list_by_flag_votes(*args):

		return Customer.Clayful.call_api({
			'model_name':       Customer.name,
			'method_name':      'list_by_flag_votes',
			'http_method':      'GET',
			'path':             '/v1/{voteModel}/{voteModelId}/flags/customers',
			'params':           ('voteModel', 'voteModelId', ),
			'args':             args
		})

	@staticmethod
	def list_by_help_votes(*args):

		return Customer.Clayful.call_api({
			'model_name':       Customer.name,
			'method_name':      'list_by_help_votes',
			'http_method':      'GET',
			'path':             '/v1/{voteModel}/{voteModelId}/helped/{upDown}/customers',
			'params':           ('voteModel', 'voteModelId', 'upDown', ),
			'args':             args
		})

	@staticmethod
	def list_by_help_votes(*args):

		return Customer.Clayful.call_api({
			'model_name':       Customer.name,
			'method_name':      'list_by_help_votes',
			'http_method':      'GET',
			'path':             '/v1/{voteModel}/{voteModelId}/helped/{upDown}/customers',
			'params':           ('voteModel', 'voteModelId', 'upDown', ),
			'args':             args
		})

	@staticmethod
	def create(*args):

		return Customer.Clayful.call_api({
			'model_name':       Customer.name,
			'method_name':      'create',
			'http_method':      'POST',
			'path':             '/v1/customers',
			'params':           (),
			'args':             args
		})

	@staticmethod
	def create_me(*args):

		return Customer.Clayful.call_api({
			'model_name':       Customer.name,
			'method_name':      'create_me',
			'http_method':      'POST',
			'path':             '/v1/me',
			'params':           (),
			'args':             args
		})

	@staticmethod
	def authenticate(*args):

		return Customer.Clayful.call_api({
			'model_name':       Customer.name,
			'method_name':      'authenticate',
			'http_method':      'POST',
			'path':             '/v1/customers/auth',
			'params':           (),
			'args':             args
		})

	@staticmethod
	def create_verification(*args):

		return Customer.Clayful.call_api({
			'model_name':       Customer.name,
			'method_name':      'create_verification',
			'http_method':      'POST',
			'path':             '/v1/customers/verifications',
			'params':           (),
			'args':             args
		})

	@staticmethod
	def authenticate_by_3rd_party(*args):

		return Customer.Clayful.call_api({
			'model_name':       Customer.name,
			'method_name':      'authenticate_by_3rd_party',
			'http_method':      'POST',
			'path':             '/v1/customers/auth/{vendor}',
			'params':           ('vendor', ),
			'without_payload':  True,
			'args':             args
		})

	@staticmethod
	def request_verification_email(*args):

		return Customer.Clayful.call_api({
			'model_name':       Customer.name,
			'method_name':      'request_verification_email',
			'http_method':      'POST',
			'path':             '/v1/customers/verifications/emails',
			'params':           (),
			'args':             args
		})

	@staticmethod
	def request_verification(*args):

		return Customer.Clayful.call_api({
			'model_name':       Customer.name,
			'method_name':      'request_verification',
			'http_method':      'POST',
			'path':             '/v1/customers/verifications/{channelSlug}',
			'params':           ('channelSlug', ),
			'args':             args
		})

	@staticmethod
	def verify(*args):

		return Customer.Clayful.call_api({
			'model_name':       Customer.name,
			'method_name':      'verify',
			'http_method':      'POST',
			'path':             '/v1/customers/{customerId}/verified',
			'params':           ('customerId', ),
			'args':             args
		})

	@staticmethod
	def add_coupon(*args):

		return Customer.Clayful.call_api({
			'model_name':       Customer.name,
			'method_name':      'add_coupon',
			'http_method':      'POST',
			'path':             '/v1/customers/{customerId}/coupons',
			'params':           ('customerId', ),
			'args':             args
		})

	@staticmethod
	def push_to_metafield(*args):

		return Customer.Clayful.call_api({
			'model_name':       Customer.name,
			'method_name':      'push_to_metafield',
			'http_method':      'POST',
			'path':             '/v1/customers/{customerId}/meta/{field}/push',
			'params':           ('customerId', 'field', ),
			'args':             args
		})

	@staticmethod
	def increase_metafield(*args):

		return Customer.Clayful.call_api({
			'model_name':       Customer.name,
			'method_name':      'increase_metafield',
			'http_method':      'POST',
			'path':             '/v1/customers/{customerId}/meta/{field}/inc',
			'params':           ('customerId', 'field', ),
			'args':             args
		})

	@staticmethod
	def pull_from_metafield(*args):

		return Customer.Clayful.call_api({
			'model_name':       Customer.name,
			'method_name':      'pull_from_metafield',
			'http_method':      'POST',
			'path':             '/v1/customers/{customerId}/meta/{field}/pull',
			'params':           ('customerId', 'field', ),
			'args':             args
		})

	@staticmethod
	def update_me(*args):

		return Customer.Clayful.call_api({
			'model_name':       Customer.name,
			'method_name':      'update_me',
			'http_method':      'PUT',
			'path':             '/v1/me',
			'params':           (),
			'args':             args
		})

	@staticmethod
	def update(*args):

		return Customer.Clayful.call_api({
			'model_name':       Customer.name,
			'method_name':      'update',
			'http_method':      'PUT',
			'path':             '/v1/customers/{customerId}',
			'params':           ('customerId', ),
			'args':             args
		})

	@staticmethod
	def update_credentials_for_me(*args):

		return Customer.Clayful.call_api({
			'model_name':       Customer.name,
			'method_name':      'update_credentials_for_me',
			'http_method':      'PUT',
			'path':             '/v1/me/credentials',
			'params':           (),
			'args':             args
		})

	@staticmethod
	def update_credentials(*args):

		return Customer.Clayful.call_api({
			'model_name':       Customer.name,
			'method_name':      'update_credentials',
			'http_method':      'PUT',
			'path':             '/v1/customers/{customerId}/credentials',
			'params':           ('customerId', ),
			'args':             args
		})

	@staticmethod
	def reset_password(*args):

		return Customer.Clayful.call_api({
			'model_name':       Customer.name,
			'method_name':      'reset_password',
			'http_method':      'PUT',
			'path':             '/v1/customers/{customerId}/password',
			'params':           ('customerId', ),
			'args':             args
		})

	@staticmethod
	def delete_me(*args):

		return Customer.Clayful.call_api({
			'model_name':       Customer.name,
			'method_name':      'delete_me',
			'http_method':      'DELETE',
			'path':             '/v1/me',
			'params':           (),
			'args':             args
		})

	@staticmethod
	def delete(*args):

		return Customer.Clayful.call_api({
			'model_name':       Customer.name,
			'method_name':      'delete',
			'http_method':      'DELETE',
			'path':             '/v1/customers/{customerId}',
			'params':           ('customerId', ),
			'args':             args
		})

	@staticmethod
	def delete_coupon_for_me(*args):

		return Customer.Clayful.call_api({
			'model_name':       Customer.name,
			'method_name':      'delete_coupon_for_me',
			'http_method':      'DELETE',
			'path':             '/v1/me/coupons/{couponId}',
			'params':           ('couponId', ),
			'args':             args
		})

	@staticmethod
	def delete_coupon(*args):

		return Customer.Clayful.call_api({
			'model_name':       Customer.name,
			'method_name':      'delete_coupon',
			'http_method':      'DELETE',
			'path':             '/v1/customers/{customerId}/coupons/{couponId}',
			'params':           ('customerId', 'couponId', ),
			'args':             args
		})

	@staticmethod
	def delete_metafield(*args):

		return Customer.Clayful.call_api({
			'model_name':       Customer.name,
			'method_name':      'delete_metafield',
			'http_method':      'DELETE',
			'path':             '/v1/customers/{customerId}/meta/{field}',
			'params':           ('customerId', 'field', ),
			'args':             args
		})




from setuptools import setup, find_packages

setup(
	name                          = 'clayful',
	version                       = '2.2.0',
	description                   = 'Python SDK for Clayful API',
	long_description              = open('README.md').read(),
	long_description_content_type = 'text/markdown',
	author                        = 'Daeik Kim',
	author_email                  = 'daeik.kim@clayful.io',
	url                           = 'https://github.com/Clayful/clayful-python',
	keywords                      = 'eCommerce clayful',
	license                       = 'MIT',
	packages                      = find_packages(),
	install_requires              = [
		'requests>2.1,<3'
	],
	extras_require   = {
		'test': ['httpretty>0.8']
	}
)


import requests
from clayful.response import ClayfulResponse
from clayful.exception import ClayfulException

def request(request_options):

	options = {
		'params':  request_options.get('query', {}),
		'headers': request_options.get('headers', {}),
	}

	if request_options.get('payload') != None:

		if request_options.get('uses_form_data') == True:

			options['files'] = request_options['payload'] # multipart/form-data

		else:

			options['json'] = request_options['payload']  # application/json

	result = requests.request(
		request_options['http_method'],
		request_options['request_url'],
		**options
	)

	data = None

	try:
		data = result.json()

	except ValueError:
		data = None

	if result.status_code >= 400:

		raise ClayfulException(
			request_options.get('model_name'),
			request_options.get('method_name'),
			result.status_code,
			result.headers,
			data.get('errorCode', None),
			data.get('message', None),
			data.get('validation', None)
		)

	else:

		return ClayfulResponse(result.status_code, data, result.headers)



class Subscription:

	Clayful = None
	name = 'Subscription'
	path = 'subscriptions'

	@staticmethod
	def config(clayful):

		Subscription.Clayful = clayful

		return Subscription

	@staticmethod
	def list(*args):

		return Subscription.Clayful.call_api({
			'model_name':       Subscription.name,
			'method_name':      'list',
			'http_method':      'GET',
			'path':             '/v1/subscriptions',
			'params':           (),
			'args':             args
		})

	@staticmethod
	def list_for_me(*args):

		return Subscription.Clayful.call_api({
			'model_name':       Subscription.name,
			'method_name':      'list_for_me',
			'http_method':      'GET',
			'path':             '/v1/me/subscriptions',
			'params':           (),
			'args':             args
		})

	@staticmethod
	def count(*args):

		return Subscription.Clayful.call_api({
			'model_name':       Subscription.name,
			'method_name':      'count',
			'http_method':      'GET',
			'path':             '/v1/subscriptions/count',
			'params':           (),
			'args':             args
		})

	@staticmethod
	def get(*args):

		return Subscription.Clayful.call_api({
			'model_name':       Subscription.name,
			'method_name':      'get',
			'http_method':      'GET',
			'path':             '/v1/subscriptions/{subscriptionId}',
			'params':           ('subscriptionId', ),
			'args':             args
		})

	@staticmethod
	def count_for_me(*args):

		return Subscription.Clayful.call_api({
			'model_name':       Subscription.name,
			'method_name':      'count_for_me',
			'http_method':      'GET',
			'path':             '/v1/me/subscriptions/count',
			'params':           (),
			'args':             args
		})

	@staticmethod
	def get_for_me(*args):

		return Subscription.Clayful.call_api({
			'model_name':       Subscription.name,
			'method_name':      'get_for_me',
			'http_method':      'GET',
			'path':             '/v1/me/subscriptions/{subscriptionId}',
			'params':           ('subscriptionId', ),
			'args':             args
		})

	@staticmethod
	def list_inventory_operations(*args):

		return Subscription.Clayful.call_api({
			'model_name':       Subscription.name,
			'method_name':      'list_inventory_operations',
			'http_method':      'GET',
			'path':             '/v1/subscriptions/{subscriptionId}/inventory/operations',
			'params':           ('subscriptionId', ),
			'args':             args
		})

	@staticmethod
	def mark_as_done(*args):

		return Subscription.Clayful.call_api({
			'model_name':       Subscription.name,
			'method_name':      'mark_as_done',
			'http_method':      'POST',
			'path':             '/v1/subscriptions/{subscriptionId}/done',
			'params':           ('subscriptionId', ),
			'without_payload':  True,
			'args':             args
		})

	@staticmethod
	def authenticate(*args):

		return Subscription.Clayful.call_api({
			'model_name':       Subscription.name,
			'method_name':      'authenticate',
			'http_method':      'POST',
			'path':             '/v1/subscriptions/{subscriptionId}/auth',
			'params':           ('subscriptionId', ),
			'args':             args
		})

	@staticmethod
	def sync_inventory(*args):

		return Subscription.Clayful.call_api({
			'model_name':       Subscription.name,
			'method_name':      'sync_inventory',
			'http_method':      'POST',
			'path':             '/v1/subscriptions/{subscriptionId}/synced',
			'params':           ('subscriptionId', ),
			'without_payload':  True,
			'args':             args
		})

	@staticmethod
	def schedule(*args):

		return Subscription.Clayful.call_api({
			'model_name':       Subscription.name,
			'method_name':      'schedule',
			'http_method':      'POST',
			'path':             '/v1/subscriptions/{subscriptionId}/scheduled',
			'params':           ('subscriptionId', ),
			'args':             args
		})

	@staticmethod
	def cancel(*args):

		return Subscription.Clayful.call_api({
			'model_name':       Subscription.name,
			'method_name':      'cancel',
			'http_method':      'POST',
			'path':             '/v1/subscriptions/{subscriptionId}/cancellation',
			'params':           ('subscriptionId', ),
			'args':             args
		})

	@staticmethod
	def cancel_for_me(*args):

		return Subscription.Clayful.call_api({
			'model_name':       Subscription.name,
			'method_name':      'cancel_for_me',
			'http_method':      'POST',
			'path':             '/v1/me/subscriptions/{subscriptionId}/cancellation',
			'params':           ('subscriptionId', ),
			'args':             args
		})

	@staticmethod
	def schedule_for_me(*args):

		return Subscription.Clayful.call_api({
			'model_name':       Subscription.name,
			'method_name':      'schedule_for_me',
			'http_method':      'POST',
			'path':             '/v1/me/subscriptions/{subscriptionId}/scheduled',
			'params':           ('subscriptionId', ),
			'args':             args
		})

	@staticmethod
	def fulfill_schedule(*args):

		return Subscription.Clayful.call_api({
			'model_name':       Subscription.name,
			'method_name':      'fulfill_schedule',
			'http_method':      'POST',
			'path':             '/v1/subscriptions/{subscriptionId}/schedules/orders',
			'params':           ('subscriptionId', ),
			'args':             args
		})

	@staticmethod
	def pull_from_metafield(*args):

		return Subscription.Clayful.call_api({
			'model_name':       Subscription.name,
			'method_name':      'pull_from_metafield',
			'http_method':      'POST',
			'path':             '/v1/subscriptions/{subscriptionId}/meta/{field}/pull',
			'params':           ('subscriptionId', 'field', ),
			'args':             args
		})

	@staticmethod
	def increase_metafield(*args):

		return Subscription.Clayful.call_api({
			'model_name':       Subscription.name,
			'method_name':      'increase_metafield',
			'http_method':      'POST',
			'path':             '/v1/subscriptions/{subscriptionId}/meta/{field}/inc',
			'params':           ('subscriptionId', 'field', ),
			'args':             args
		})

	@staticmethod
	def push_to_metafield(*args):

		return Subscription.Clayful.call_api({
			'model_name':       Subscription.name,
			'method_name':      'push_to_metafield',
			'http_method':      'POST',
			'path':             '/v1/subscriptions/{subscriptionId}/meta/{field}/push',
			'params':           ('subscriptionId', 'field', ),
			'args':             args
		})

	@staticmethod
	def update(*args):

		return Subscription.Clayful.call_api({
			'model_name':       Subscription.name,
			'method_name':      'update',
			'http_method':      'PUT',
			'path':             '/v1/subscriptions/{subscriptionId}',
			'params':           ('subscriptionId', ),
			'args':             args
		})

	@staticmethod
	def update_for_me(*args):

		return Subscription.Clayful.call_api({
			'model_name':       Subscription.name,
			'method_name':      'update_for_me',
			'http_method':      'PUT',
			'path':             '/v1/me/subscriptions/{subscriptionId}',
			'params':           ('subscriptionId', ),
			'args':             args
		})

	@staticmethod
	def update_cancellation(*args):

		return Subscription.Clayful.call_api({
			'model_name':       Subscription.name,
			'method_name':      'update_cancellation',
			'http_method':      'PUT',
			'path':             '/v1/subscriptions/{subscriptionId}/cancellation',
			'params':           ('subscriptionId', ),
			'args':             args
		})

	@staticmethod
	def update_cancellation_for_me(*args):

		return Subscription.Clayful.call_api({
			'model_name':       Subscription.name,
			'method_name':      'update_cancellation_for_me',
			'http_method':      'PUT',
			'path':             '/v1/me/subscriptions/{subscriptionId}/cancellation',
			'params':           ('subscriptionId', ),
			'args':             args
		})

	@staticmethod
	def delete(*args):

		return Subscription.Clayful.call_api({
			'model_name':       Subscription.name,
			'method_name':      'delete',
			'http_method':      'DELETE',
			'path':             '/v1/subscriptions/{subscriptionId}',
			'params':           ('subscriptionId', ),
			'args':             args
		})

	@staticmethod
	def mark_as_undone(*args):

		return Subscription.Clayful.call_api({
			'model_name':       Subscription.name,
			'method_name':      'mark_as_undone',
			'http_method':      'DELETE',
			'path':             '/v1/subscriptions/{subscriptionId}/done',
			'params':           ('subscriptionId', ),
			'args':             args
		})

	@staticmethod
	def delete_metafield(*args):

		return Subscription.Clayful.call_api({
			'model_name':       Subscription.name,
			'method_name':      'delete_metafield',
			'http_method':      'DELETE',
			'path':             '/v1/subscriptions/{subscriptionId}/meta/{field}',
			'params':           ('subscriptionId', 'field', ),
			'args':             args
		})

	@staticmethod
	def delete_inventory_operation(*args):

		return Subscription.Clayful.call_api({
			'model_name':       Subscription.name,
			'method_name':      'delete_inventory_operation',
			'http_method':      'DELETE',
			'path':             '/v1/subscriptions/{subscriptionId}/inventory/operations/{operationId}',
			'params':           ('subscriptionId', 'operationId', ),
			'args':             args
		})




class Cart:

	Clayful = None
	name = 'Cart'
	path = ''

	@staticmethod
	def config(clayful):

		Cart.Clayful = clayful

		return Cart

	@staticmethod
	def count_items_for_me(*args):

		return Cart.Clayful.call_api({
			'model_name':       Cart.name,
			'method_name':      'count_items_for_me',
			'http_method':      'GET',
			'path':             '/v1/me/cart/items/count',
			'params':           (),
			'args':             args
		})

	@staticmethod
	def count_items(*args):

		return Cart.Clayful.call_api({
			'model_name':       Cart.name,
			'method_name':      'count_items',
			'http_method':      'GET',
			'path':             '/v1/customers/{customerId}/cart/items/count',
			'params':           ('customerId', ),
			'args':             args
		})

	@staticmethod
	def get_for_me(*args):

		return Cart.Clayful.call_api({
			'model_name':       Cart.name,
			'method_name':      'get_for_me',
			'http_method':      'POST',
			'path':             '/v1/me/cart',
			'params':           (),
			'args':             args
		})

	@staticmethod
	def get_as_non_registered(*args):

		return Cart.Clayful.call_api({
			'model_name':       Cart.name,
			'method_name':      'get_as_non_registered',
			'http_method':      'POST',
			'path':             '/v1/customers/non-registered/cart',
			'params':           (),
			'args':             args
		})

	@staticmethod
	def get(*args):

		return Cart.Clayful.call_api({
			'model_name':       Cart.name,
			'method_name':      'get',
			'http_method':      'POST',
			'path':             '/v1/customers/{customerId}/cart',
			'params':           ('customerId', ),
			'args':             args
		})

	@staticmethod
	def add_item_for_me(*args):

		return Cart.Clayful.call_api({
			'model_name':       Cart.name,
			'method_name':      'add_item_for_me',
			'http_method':      'POST',
			'path':             '/v1/me/cart/items',
			'params':           (),
			'args':             args
		})

	@staticmethod
	def get_as_non_registered_for_me(*args):

		return Cart.Clayful.call_api({
			'model_name':       Cart.name,
			'method_name':      'get_as_non_registered_for_me',
			'http_method':      'POST',
			'path':             '/v1/me/non-registered/cart',
			'params':           (),
			'args':             args
		})

	@staticmethod
	def add_item(*args):

		return Cart.Clayful.call_api({
			'model_name':       Cart.name,
			'method_name':      'add_item',
			'http_method':      'POST',
			'path':             '/v1/customers/{customerId}/cart/items',
			'params':           ('customerId', ),
			'args':             args
		})

	@staticmethod
	def checkout_for_me(*args):

		return Cart.Clayful.call_api({
			'model_name':       Cart.name,
			'method_name':      'checkout_for_me',
			'http_method':      'POST',
			'path':             '/v1/me/cart/checkout/{type}',
			'params':           ('type', ),
			'args':             args
		})

	@staticmethod
	def checkout_as_non_registered(*args):

		return Cart.Clayful.call_api({
			'model_name':       Cart.name,
			'method_name':      'checkout_as_non_registered',
			'http_method':      'POST',
			'path':             '/v1/customers/non-registered/cart/checkout/{type}',
			'params':           ('type', ),
			'args':             args
		})

	@staticmethod
	def checkout(*args):

		return Cart.Clayful.call_api({
			'model_name':       Cart.name,
			'method_name':      'checkout',
			'http_method':      'POST',
			'path':             '/v1/customers/{customerId}/cart/checkout/{type}',
			'params':           ('customerId', 'type', ),
			'args':             args
		})

	@staticmethod
	def checkout_as_non_registered_for_me(*args):

		return Cart.Clayful.call_api({
			'model_name':       Cart.name,
			'method_name':      'checkout_as_non_registered_for_me',
			'http_method':      'POST',
			'path':             '/v1/me/non-registered/cart/checkout/{type}',
			'params':           ('type', ),
			'args':             args
		})

	@staticmethod
	def update_item_for_me(*args):

		return Cart.Clayful.call_api({
			'model_name':       Cart.name,
			'method_name':      'update_item_for_me',
			'http_method':      'PUT',
			'path':             '/v1/me/cart/items/{itemId}',
			'params':           ('itemId', ),
			'args':             args
		})

	@staticmethod
	def update_item(*args):

		return Cart.Clayful.call_api({
			'model_name':       Cart.name,
			'method_name':      'update_item',
			'http_method':      'PUT',
			'path':             '/v1/customers/{customerId}/cart/items/{itemId}',
			'params':           ('customerId', 'itemId', ),
			'args':             args
		})

	@staticmethod
	def empty_for_me(*args):

		return Cart.Clayful.call_api({
			'model_name':       Cart.name,
			'method_name':      'empty_for_me',
			'http_method':      'DELETE',
			'path':             '/v1/me/cart/items',
			'params':           (),
			'args':             args
		})

	@staticmethod
	def empty(*args):

		return Cart.Clayful.call_api({
			'model_name':       Cart.name,
			'method_name':      'empty',
			'http_method':      'DELETE',
			'path':             '/v1/customers/{customerId}/cart/items',
			'params':           ('customerId', ),
			'args':             args
		})

	@staticmethod
	def delete_item_for_me(*args):

		return Cart.Clayful.call_api({
			'model_name':       Cart.name,
			'method_name':      'delete_item_for_me',
			'http_method':      'DELETE',
			'path':             '/v1/me/cart/items/{itemId}',
			'params':           ('itemId', ),
			'args':             args
		})

	@staticmethod
	def delete_item(*args):

		return Cart.Clayful.call_api({
			'model_name':       Cart.name,
			'method_name':      'delete_item',
			'http_method':      'DELETE',
			'path':             '/v1/customers/{customerId}/cart/items/{itemId}',
			'params':           ('customerId', 'itemId', ),
			'args':             args
		})

